from threading import Thread
import time
# Remote imports
from protonvpn_cli.utils import(
    get_config_value, 
    is_connected, 
    set_config_value #noqa
)    
from protonvpn_cli.country_codes import country_codes

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import GObject as gobject

# Local imports
from ..views.settings_view import SettingsView
from ..gui_logger import gui_logger
from ..constants import HELP_TEXT, UI_DASHBOARD, UI_SETTINGS, VERSION
from ..utils import (
    get_gui_config,
    tab_style_manager,
)

class DashboardView:
    def __init__(self, interface, Gtk, dashboard_presenter, settings_view, queue):
        interface.add_from_file(UI_DASHBOARD)
        self.set_objects(interface, Gtk, dashboard_presenter, settings_view, queue)

        interface.connect_signals({
            "profile_quick_connect_button_clicked": self.profile_quick_connect_button_clicked,
            "last_connect_button_clicked": self.last_connect_button_clicked,
            "profile_random_connect_button_clicked": self.profile_random_connect_button_clicked,
            "disconnect_button_clicked": self.disconnect_button_clicked,
            "dashboard_notebook_page_changed": self.dashboard_notebook_page_changed,
            "TreeViewServerList_cursor_changed": self.TreeViewServerList_cursor_changed,
            "main_conn_disc_button_label": self.main_conn_disc_button_label,
            "secure_core_switch_changed": self.secure_core_switch_changed,
            "manage_profiles_button_clicked": self.manage_profiles_button_clicked,
            "delete_active_profile_button_clicked": self.delete_active_profile_button_clicked,
            "server_filter_input_key_release": self.server_filter_input_key_release,
            "configuration_menu_button_clicked": self.configuration_menu_button_clicked,
            "about_menu_button_clicked": self.about_menu_button_clicked,
            "check_for_updates_button_clicked": self.check_for_updates_button_clicked,
            "diagnose_menu_button_clicked": self.diagnose_menu_button_clicked,
            "help_button_clicked": self.help_button_clicked,
            "AboutDialog_delete_event": self.AboutDialog_delete_event,
            "refresh_servers_button_clicked": self.refresh_servers_button_clicked,
            "exit_button_clicked": self.exit_button_clicked,
        })

    def display_window(self):
        self.dashboard_window.connect("destroy", self.gtk.main_quit)

        self.queue.put(dict(action="display_dialog", label="Loading...", spinner=True, hide_close_button=True))

        objects = {
            "connection_labels": self.connection_labels,
            "secure_core":{
                "secure_core_switch":self.secure_core_switch,
                "secure_core_label_style":self.secure_core_label_style,
            },
            "server_tree_list":{
                "tree_object": self.tree_object,
            }
        }

        
        thread = Thread(target=self.dashboard_presenter.on_load, args=[objects])
        thread.daemon = True
        thread.start()

        gobject.idle_add(self.dashboard_window.show)

    def set_objects(self, interface, Gtk, dashboard_presenter, settings_view, queue):
        self.gtk = Gtk
        self.interface = interface
        self.queue = queue
        self.dashboard_presenter = dashboard_presenter
        self.dashboard_window = self.interface.get_object("DashboardWindow")
        self.settings_window = settings_view

        # Top labels
        self.time_connected_label =     self.interface.get_object("time_connected_label")
        self.protocol_label =           self.interface.get_object("protocol_label")
        self.conn_disc_button_label =   self.interface.get_object("main_conn_disc_button_label")
        self.ip_label =                 self.interface.get_object("ip_label")
        self.server_load_label =        self.interface.get_object("server_load_label")
        self.country_label =            self.interface.get_object("country_label")
        self.isp_label    =             self.interface.get_object("isp_label")
        self.data_received_label =      self.interface.get_object("data_received_label")
        self.data_sent_label =          self.interface.get_object("data_sent_label") 
        self.background_large_flag =    self.interface.get_object("background_large_flag")
        self.protonvpn_sign_green =     self.interface.get_object("protonvpn_sign_green")
        self.connection_labels = {
            "time_connected_label": self.time_connected_label,
            "protocol_label": self.protocol_label,
            "conn_disc_button_label": self.conn_disc_button_label,
            "ip_label": self.ip_label,
            "server_load_label": self.server_load_label,
            "country_label": self.country_label,
            "isp_label": self.isp_label,
            "data_received_label": self.data_received_label,
            "data_sent_label": self.data_sent_label,
            "background_large_flag": self.background_large_flag,
            "protonvpn_sign_green": self.protonvpn_sign_green,
        },
        # Secure core
        self.secure_core_switch = self.interface.get_object("secure_core_switch")
        self.secure_core_label_style = self.interface.get_object("secure_core_label")
        
        # Server List
        self.tree_object = self.interface.get_object("ServerTreeStore")


        self.dashboard_tab_dict = {
            "countries_tab_style": self.interface.get_object("countries_tab_label").get_style_context(),
            "profiles_tab_style": self.interface.get_object("profiles_tab_label").get_style_context()
        }

    def profile_quick_connect_button_clicked(self, button):
        """Button/Event handler to connect to the fastest server
        """
        self.queue.put(dict(action="display_dialog", label="Connecting to the fastest server...", spinner=True, hide_close_button=True))
        gui_logger.debug(">>> Starting \"quick_connect\" thread.")

        thread = Thread(target=self.dashboard_presenter.quick_connect, kwargs=dict(connection_labels=self.connection_labels, profile_quick_connect=True)) 
        thread.daemon = True
        thread.start()

    def last_connect_button_clicked(self, button):
        """Button/Event handler to reconnect to previously connected server
        """   
        try:
            servername = get_config_value("metadata", "connected_server")
            protocol = get_config_value("metadata", "connected_proto")     
        except KeyError:
            self.queue.put(dict(action="display_dialog", label="You have not previously connected to any server, please do first connect to a server before attempting to reconnect."))
            gui_logger.debug("[!] Attempted to connect to previously connected server without having made any previous connections.")
            return

        self.queue.put(dict(action="display_dialog", label="Connecting to previously connected server <b>{0}</b> with <b>{1}</b>.".format(servername, protocol.upper()), spinner=True, hide_close_button=True))

        gui_logger.debug(">>> Starting \"last_connect\" thread.")

        thread = Thread(target=self.dashboard_presenter.on_last_connect, kwargs=dict(connection_labels=self.connection_labels))
        thread.daemon = True
        thread.start()

    def profile_random_connect_button_clicked(self, button):
        """Button/Event handler to connect to a random server
        """
        self.queue.put(dict(action="display_dialog", label="Connecting to a random server...", spinner=True, hide_close_button=True))

        gui_logger.debug(">>> Starting \"random_connect\" thread.")

        thread = Thread(target=self.dashboard_presenter.random_connect, kwargs=dict(connection_labels=self.connection_labels))
        thread.daemon = True
        thread.start()

    def refresh_servers_button_clicked(self, button):
        gui_logger.debug(">>> Starting \"on_refresh_servers\" thread.")

        self.queue.put(dict(action="display_dialog", label="Fetching servers...", spinner=True, hide_close_button=True))
        
        thread = Thread(target=self.dashboard_presenter.on_refresh_servers, kwargs=dict(tree_object=self.tree_object))
        thread.daemon = True
        thread.start()

    def disconnect_button_clicked(self, button):
        """Button/Event handler to disconnect any existing connections
        """
        self.queue.put(dict(action="display_dialog", label="Disconnecting...", spinner=True, hide_close_button=True))

        gui_logger.debug(">>> Starting \"disconnect\" thread.")

        thread = Thread(target=self.dashboard_presenter.on_disconnect, kwargs=dict(connection_labels=self.connection_labels))
        thread.daemon = True
        thread.start()

    def dashboard_notebook_page_changed(self, notebook, selected_tab, actual_tab_index):
        """Updates Dashboard Window tab style
        """
        if actual_tab_index == 1:
            tab_style_manager("profiles_tab_style", self.dashboard_tab_dict)
        else:
            tab_style_manager("countries_tab_style", self.dashboard_tab_dict)

    def TreeViewServerList_cursor_changed(self, treeview):
        """Updates Quick Connect label in the Dashabord, based on what server or contry a user clicked.
        """
        # Get the selected server
        (model, pathlist) = treeview.get_selection().get_selected_rows()

        for path in pathlist :
            tree_iter = model.get_iter(path)
            # the second param of get_value() specifies the column number, starting at 0
            user_selected_server = model.get_value(tree_iter, 1)

        try:
            self.conn_disc_button_label.set_markup("Connect to {}".format(user_selected_server))
        except UnboundLocalError:
            self.conn_disc_button_label.set_markup("Quick Connect")

    def main_conn_disc_button_label(self, button):
        """Button/Event handler to connect to either pre-selected quick connect, selected server/country or just fastest connect in the absence
        of previous params.
        """
        gui_logger.debug(">>> Starting \"main_conn_disc_button_label\" thread.")
        
        server_list = self.interface.get_object("TreeViewServerList").get_selection() 
        (model, pathlist) = server_list.get_selected_rows()

        user_selected_server = False

        for path in pathlist :
            tree_iter = model.get_iter(path)
            # the second param of get_value() specifies the column number, starting at 0
            user_selected_server = model.get_value(tree_iter, 1)

        server_list.unselect_all()

        target = self.dashboard_presenter.quick_connect 
        message = "Connecting to the fastest server..."

        if is_connected() and not user_selected_server:
            target = self.dashboard_presenter.on_disconnect
            message = "Disconnecting..."

        if user_selected_server:
            target = self.dashboard_presenter.on_connect_user_selected
            message = "Connecting to <b>{}</b>".format(user_selected_server) 
        
        self.queue.put(dict(action="display_dialog", label=message, spinner=True, hide_close_button=True))

        thread = Thread(target=target, kwargs=dict(
                                            connection_labels=self.connection_labels, 
                                            user_selected_server=user_selected_server))
        thread.daemon = True
        thread.start()

    def secure_core_switch_changed(self, switch, state):
        display_secure_core = get_gui_config("connections", "display_secure_core")
 
        if display_secure_core == "False":
            update_to = "True"
            self.secure_core_label_style.get_style_context().remove_class("disabled_label")
        else:
            update_to = "False"
            self.secure_core_label_style.get_style_context().add_class("disabled_label")
        
        if (state and display_secure_core == "False") or (not state and display_secure_core != "False"):
            self.queue.put(dict(action="display_dialog", label="Loading {} servers...".format("secure-core" if update_to == "True" else "non secure-core"), spinner=True, hide_close_button=True))
            thread = Thread(target=self.dashboard_presenter.reload_secure_core_servers, kwargs=dict(
                                                    tree_object=self.tree_object,
                                                    update_to=update_to))
            thread.daemon = True
            thread.start()
    
    def manage_profiles_button_clicked(self, button):
        self.queue.put(dict(action="display_dialog", label="This feature is not yet implemented."))
        
    def delete_active_profile_button_clicked(self, button):
        self.queue.put(dict(action="display_dialog", label="This feature is not yet implemented."))

    def server_filter_input_key_release(self, entry, event):
        """Event handler, to filter servers after each key release
        """
        user_filter_by = entry.get_text()
        server_tree_store = self.interface.get_object("ServerTreeStore")
        tree_view_object = self.interface.get_object("TreeViewServerList")

        # Creates a new filter from a ListStore/TreeStore
        n_filter = server_tree_store.filter_new()

        # set_visible_func:
        # first_param: filter function
        # seconde_param: input to filter by
        n_filter.set_visible_func(self.column_filter, data=user_filter_by)
        
        # Apply the filter model to a TreeView
        tree_view_object.set_model(n_filter)

        # Updates the ListStore model
        n_filter.refilter()

    def column_filter(self, model, iterator, data=None):
        """Filter by columns and returns the corresponding rows
        """
        treeview = self.interface.get_object("TreeViewServerList")
        
        for col in range(0, treeview.get_n_columns()):
            value = model.get_value(iterator, col)
            if isinstance(value, str):
                if data.lower() in value.lower():
                    return True

    def configuration_menu_button_clicked(self, button):
        """Button/Event handler to open Configurations window
        """
        gui_logger.debug(">>> Starting \"load_configurations\".")
        gobject.idle_add(self.settings_window.display_window)

    def about_menu_button_clicked(self, button):
        """Button /Event handler to open About dialog
        """
        about_dialog = self.interface.get_object("AboutDialog")
        about_dialog.set_version("v."+VERSION)
        gobject.idle_add(about_dialog.show)

    def check_for_updates_button_clicked(self, button):
        """Button/Event handler to check for update.
        """
        self.queue.put(dict(action="display_dialog", label="Checking...", spinner=True, hide_close_button=True))

        gui_logger.debug(">>> Starting \"message_dialog\" thread. [CHECK_FOR_UPDATES]")

        thread = Thread(target=self.dashboard_presenter.on_check_for_updates)
        thread.daemon = True
        thread.start()

    def diagnose_menu_button_clicked(self, button):
        """Button/Event handler top show diagnose window.
        """
        self.queue.put(dict(action="display_dialog", label="Diagnosing...", spinner=True, hide_close_button=True))

        gui_logger.debug(">>> Starting \"message_dialog\" thread. [DIAGNOSE]")
        thread = Thread(target=self.dashboard_presenter.on_diagnose)
        thread.daemon = True
        thread.start()

    def help_button_clicked(self, button):
        """Button/Event handler to show help information.
        """
        self.queue.put(dict(action="display_dialog", label=HELP_TEXT))

    # To avoid getting the AboutDialog destroyed and not being re-rendered again
    def AboutDialog_delete_event(self, window, event):
        """On Delete handler is used to hide the dialog and so that it successfully renders next time it is called
        
        -Returns:Boolean
        - It needs to return True, otherwise the content will not re-render after closing the window
        """
        if window.get_property("visible") is True:
            window.hide()
            return True    
    
    def exit_button_clicked(self, button):
        self.gtk.main_quit()



import os
import re
import subprocess

# Remote imports
from protonvpn_cli.constants import CONFIG_DIR, PASSFILE, SPLIT_TUNNEL_FILE, USER #noqa
from protonvpn_cli.utils import get_config_value, is_valid_ip, set_config_value, get_servers, get_country_name #noqa
from protonvpn_cli.connection import disconnect as pvpn_disconnect
from protonvpn_cli.country_codes import country_codes

from ..gui_logger import gui_logger
from ..constants import (
    TRAY_CFG_DICT, 
    TEMPLATE,
    PATH_AUTOCONNECT_SERVICE,
    SERVICE_NAME,
    TRAY_SUDO_TYPES,
)
from ..utils import (
    set_gui_config,
    get_gui_config,
    find_cli,
)

class SettingsService:
    sudo_timeout = 10

    def set_user_pass(self, username, password):
        user_pass = "'{}\n{}'".format(username, password)
        echo_to_passfile = "echo -e {} > {}".format(user_pass, PASSFILE)

        result_bool, display_message = self.root_command(["bash", "-c", echo_to_passfile])

        if not result_bool:
            gui_logger.debug("Could not update Passfile")
            return result_bool, display_message

        set_config_value("USER", "username", username)
        
        return result_bool, "Username and password <b>updated</b>!"

    def set_dns(self, dns_value):
        try:
            set_config_value("USER", "dns_leak_protection", dns_value)
        except:
            gui_logger.debug("Could not update DNS Protection settings to: {}".format(dns_value))
            return False

        return True

    def set_pvpn_tier(self, protonvpn_plan):
        visionary_compare = 0

        visionary_compare = int(protonvpn_plan)
        if protonvpn_plan == 4:
            protonvpn_plan = 3

        # Lower tier by one to match API allocation
        protonvpn_plan -= 1    

        try:
            set_config_value("USER", "tier", str(protonvpn_plan))
        except:
            gui_logger.debug("Could not update ProtonVPN Plan to: {}".format(protonvpn_plan))
            return False

        return True
    
    def set_default_protocol(self, protocol):
        try:
            set_config_value("USER", "default_protocol", protocol)
        except:
            gui_logger.debug("Could not update default Protocol to: {}".format(protocol))
            return False

        return True

    def set_autoconnect(self, update_to):
            # autoconnect_alternatives = ["dis", "fast", "rand", "p2p", "sc", "tor"]
            return_val = False
            return_bool, return_msg = self.manage_autoconnect(mode="disable")

            if update_to == "dis":
                pass
            elif update_to == "fast":
                return_bool, return_msg = self.manage_autoconnect(mode="enable", command="connect -f")
            elif update_to == "rand":
                return_bool, return_msg = self.manage_autoconnect(mode="enable", command="connect -r")
            elif update_to == "p2p":
                return_bool, return_msg = self.manage_autoconnect(mode="enable", command="connect --p2p")
            elif update_to == "sc":
                return_bool, return_msg = self.manage_autoconnect(mode="enable", command="connect --sc")
            elif update_to == "tor":
                return_bool, return_msg = self.manage_autoconnect(mode="enable", command="connect --tor")
            else:
                # Connect to a specific country
                if update_to in country_codes:
                    return_bool, return_msg = self.manage_autoconnect(mode="enable", command="connect --cc " + update_to.upper())

            if not return_bool:
                return False, return_msg

            try:
                set_gui_config("conn_tab", "autoconnect", update_to)
            except:
                gui_logger.debug("Could not update autoconnect in pvpn-gui.cfg to: {}".format(update_to))
                return False, "Unable to update autoconnect configurations although autoconnect should be enabled"

            return True, return_msg

    def set_quickconnect(self, update_to):
        return_val = False

        if update_to in country_codes or update_to in ["dis", "fast", "rand", "p2p", "sc", "tor"]:
            try:
                set_gui_config("conn_tab", "quick_connect", update_to)
                return_val = True
            except:
                gui_logger.debug("Could not update quickconnect to: {}".format(update_to))
                return False

        return return_val

    def set_killswitch(self, update_to):
        try:
            set_config_value("USER", "killswitch", update_to)
        except:
            gui_logger.debug("Could not update KillSwitch to: {}".format(update_to))
            return False

        return True

    def set_split_tunneling(self, update_to):
        if not update_to == "1":
            try:
                if os.path.isfile(SPLIT_TUNNEL_FILE):
                    os.remove(SPLIT_TUNNEL_FILE)
            except:
                return False

        try:
            if int(get_config_value("USER", "killswitch")):
                set_config_value("USER", "killswitch", 0)
        except:
            gui_logger.debug("Could not disable KillSwitch")
            return False

        try:
            set_config_value("USER", "split_tunnel", update_to)
        except:
            gui_logger.debug("Could not update split_tunnel to: {}".format(update_to))
            return False

    def set_split_tunneling_ips(self, ip_list):
        try:
            with open(SPLIT_TUNNEL_FILE, "w") as f:
                for ip in ip_list:
                    f.write("\n{0}".format(ip))
        except:
            gui_logger.debug("Could not update add IPs to file: {}".format(ip_list))
            return False

        return True

    def check_valid_ips(self, split_tunneling_content):
        for ip in split_tunneling_content:
            if not is_valid_ip(ip):
                # dialog_window.update_dialog(label="<b>{0}</b> is not valid!\nNone of the IP's were added, please try again with a different IP.".format(ip))
                gui_logger.debug("[!] Invalid IP \"{0}\".".format(ip))
                return (False, ip)

        return True
        
    def reformat_ip_list(self, ip_list):
        # Split IP/CIDR by either ";" and/or "\n"
        split_tunneling_content = re.split('[;\n]', ip_list)
        # Remove empty spaces
        split_tunneling_content = [content.strip() for content in split_tunneling_content]
        # Remove empty list elements
        return list(filter(None, split_tunneling_content))

    def set_tray_display_setting(self, tray_display, tray_setting):
        try:
            set_gui_config("tray_tab", TRAY_CFG_DICT[tray_display], tray_setting)
        except:
            gui_logger.debug("Could not update tray configurations.\n Settings: {} --- {}".format(TRAY_CFG_DICT[tray_display], tray_setting))
            return False

        return True
    
    def update_polkit(self, update_to):
        changed_to_msg = "disabled" if update_to == 0 else "enabled"
        try:
            self.polkit = update_to
        except (KeyError, IOError) as e:
            gui_logger.debug("Unable to update PolKit configuration: {}".format(e))
            return False, "Unable to <b>{}</b> PolKit configuration!\nError: {}".format(changed_to_msg, e)

        return True, "Polkit Support <b>{}</b>".format(changed_to_msg)

    @property
    def polkit(self):
        try:
            return int(get_gui_config("general_tab", "polkit_enabled"))
        except KeyError:
            set_gui_config("general_tab", "polkit_enabled", 0)
            return 0

    @polkit.setter
    def polkit(self, update_to):
        set_gui_config("general_tab", "polkit_enabled", update_to)

    def generate_autoconnect_list(self):
        countries = {}
        servers = get_servers()
        
        autoconnect_alternatives = ["dis", "fast", "rand", "p2p", "sc", "tor"]

        for server in servers:
            country = get_country_name(server["ExitCountry"])
            if country not in countries.keys():
                countries[country] = []
            countries[country].append(server["Name"])
        
        for country in sorted(countries):
            autoconnect_alternatives.append(country)

        return autoconnect_alternatives
    
    def manage_autoconnect(self, mode, command=False):
        """Function that manages autoconnect functionality. It takes a mode (enabled/disabled) and a command that is to be passed to the CLI.
        """
        if mode == "enable":
            return self.enable_autoconnect(command)

        if mode == "disable":
            return self.disable_autoconnect()

    def enable_autoconnect(self, command):
        """Function that enables autoconnect.
        """
        timeout = False
        protonvpn_path = find_cli()
        if not protonvpn_path:
            return False, "Could not find path to CLI"

        # Injects CLIs start and stop path and username
        with_cli_path = TEMPLATE.replace("PATH", (protonvpn_path + " " + command))
        template = with_cli_path.replace("STOP", protonvpn_path + " disconnect")
        template = template.replace("=user", "="+USER)

        generate_service_command = "cat > {0} <<EOF {1}\nEOF".format(PATH_AUTOCONNECT_SERVICE, template)
        process = subprocess.Popen(self.sudo_type, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE, universal_newlines=True)

        process.stdin.write("bash -c {}\n".format(generate_service_command))
        process.stdin.write("systemctl enable {}\n".format(SERVICE_NAME))
        process.stdin.write("systemctl daemon-reload\n")
        process.stdin.flush()

        try:
            outs, errs = process.communicate(timeout=15)
        except subprocess.TimeoutExpired:
            timeout = True
            process.kill()

        errs = errs.lower()
        outs = outs.lower()
        gui_logger.debug("errs: {}\nouts: {}".format(errs, outs))

        if "dismissed" in errs and not timeout:
            return False, "Administrator access was dismissed."
        
        if not "dismissed" in errs and timeout:
            return False, "Unable to process request. Administrator access has probably not been provided.\nTo do so, please run the GUI from within a terminal or enable PolKit Support from within the settings window."

        if not "created symlink" in errs.lower():
            return False, "Unable to setup autoconnect!"

        if not self.daemon_exists():
            return False, "Could not configure autoconnect!"

        return True, "Autoconnect enabled!"
        
    def disable_autoconnect(self):
        """Function that disables autoconnect.
        """
        timeout = False
        if not self.daemon_exists():
            return True, "Autoconnect disabled!"

        process = subprocess.Popen(self.sudo_type, stdout=subprocess.PIPE, stderr=subprocess.PIPE, stdin=subprocess.PIPE, universal_newlines=True)

        process.stdin.write("systemctl stop {}\n".format(SERVICE_NAME))
        process.stdin.write("systemctl disable {}\n".format(SERVICE_NAME))
        process.stdin.write("rm {}\n".format(PATH_AUTOCONNECT_SERVICE))
        process.stdin.write("systemctl daemon-reload\n")
        process.stdin.flush()

        try:
            outs, errs = process.communicate(timeout=15)
        except subprocess.TimeoutExpired:
            timeout = True
            process.kill()
            outs, errs = process.communicate()

        errs = errs.lower()
        outs = outs.lower()
        gui_logger.debug("errs: {}\nouts: {}".format(errs, outs))

        if "dismissed" in errs and not timeout:
            return False, "Administrator access was dismissed."
        
        if not "dismissed" in errs and timeout:
            return False, "Unable to process request. Administrator access has probably not been provided."

        if "does not exist." in errs:
            return False, "Can't disable autoconnect since .service file does not exist.\nThis might be due to some misconfiguration in config file.\nPlease try to connect to another country, then disable it."
       
        if not "removed" in errs:
            return False, "Unable to disable autoconnect!"

        if self.daemon_exists():
            return False, "Autoconnect configurations still exists!"

        return True, "Autoconnect disabled!"

    def daemon_exists(self):
        """Function that checks if autoconnect daemon service exists.
        """
        # Return code 3: service exists
        # Return code 4: service could not be found
        resp_stop = subprocess.run(["systemctl", "status" , SERVICE_NAME], stdout=subprocess.PIPE, stderr=subprocess.PIPE) # nosec
        return_val = True
        gui_logger.debug(resp_stop)
        if resp_stop.returncode == 4 and not resp_stop.returncode == 3:
            return_val = False

        return return_val

    def root_command(self, command_list, enable=False):
        return_on_sucess_message = "PolKit Support <b>disabled</b>!"
        if enable:
            return_on_sucess_message = "PolKit Support <b>enabled</b>!\nYou can now use \"pkexec\" command to start ProtonVPN GUI."
        
        command_list.insert(0, self.sudo_type)

        process = subprocess.Popen(command_list, stdout=subprocess.PIPE, stderr=subprocess.PIPE) # nosec
        timeout = False

        try:
            outs, errs = process.communicate(timeout=self.sudo_timeout)
        except subprocess.TimeoutExpired:
            timeout = True
            process.kill()
            outs, errs = process.communicate()

        errs = errs.decode().lower()
        outs = outs.decode().lower()
        gui_logger.debug("errs: {}\nouts: {}".format(errs, outs))

        if "terminal is required" in errs:
            return False, "Administrator access is required, and PolKit Support is not enabled.\nPlease launch the app either from within a terminal or enable PolKit Support."

        if "dismissed" in errs and not timeout:
            return False, "Administrator access was dismissed."
        
        if not "dismissed" in errs and timeout:
            return False, "Unable to process request. Administrator access has probably not been provided.\nTo do so, please run the GUI from within a terminal or enable PolKit Support from within the settings window."

        return True, return_on_sucess_message


    def get_gui_path(self):
        """Function that searches for the CLI. Returns CLIs path if it is found, otherwise it returns False.
        """
        protonvpn_gui_path = subprocess.run(["which", "protonvpn-gui"], stdout=subprocess.PIPE, stderr=subprocess.PIPE) # nosec
        if protonvpn_gui_path.returncode == 1:
            gui_logger.debug("[!] Unable to run \"get_gui_path\" subprocess. Result: \"{}\"".format(protonvpn_gui_path))
            protonvpn_gui_path = False

        return protonvpn_gui_path.stdout.decode()[:-1] if (protonvpn_gui_path and protonvpn_gui_path.returncode == 0) else False
    
    @property
    def sudo_type(self):
        try:
            is_polkit_enabled =  int(get_gui_config("general_tab", "polkit_enabled"))
        except (KeyError, NameError):
            return "sudo"

        return_val = "sudo"

        if is_polkit_enabled == 1:
            return_val = "pkexec"

        return return_val 



import requests
import subprocess
import collections

from protonvpn_cli.country_codes import country_codes
from protonvpn_cli.utils import get_config_value, set_config_value, is_connected, get_server_value, get_country_name

from ..utils import set_gui_config, get_gui_config, check_internet_conn, get_server_protocol_from_cli
from ..constants import GITHUB_URL_RELEASE, SMALL_FLAGS_BASE_PATH, FEATURES_BASE_PATH
from ..gui_logger import gui_logger

class DashboardService:
    # 30 seconds of timeout for all root necessary commands
    timeout_value = 30

    def set_display_secure_core(self, update_to):
        try:
            set_gui_config("connections", "display_secure_core", update_to)
        except:
            gui_logger.debug("Could not update display_secure_core to: {}".format(update_to))
            return False

        return True

    def connect_to_server(self, user_selected_server):
        command = ["protonvpn", "connect", user_selected_server]
        bool_value, result =  self.root_command(command)
        return self.get_display_message(bool_value, result)

    def connect_to_country(self, user_selected_server):
        try:
            for k, v in country_codes.items():
                if v == user_selected_server:
                    selected_country = k
                    break
        except:
            return False

        command = ["protonvpn", "connect", "--cc", selected_country]
        bool_value, result =  self.root_command(command)
        return self.get_display_message(bool_value, result)

    def quick_connect_manager(self, profile_quick_connect):
        try:
            user_selected_quick_connect = get_gui_config("conn_tab","quick_connect")
        except KeyError:
            user_selected_quick_connect = False

        if user_selected_quick_connect and not user_selected_quick_connect == "dis" and not profile_quick_connect:
            return self.custom_quick_connect(user_selected_quick_connect)
        
        return self.quick_connect()

    def custom_quick_connect(self, quick_conn_pref):
        command = "--fastest"
        country = False

        if quick_conn_pref == "fast":
            command="-f"
        elif quick_conn_pref == "rand":
            command="-r"
        elif quick_conn_pref == "p2p":
            command="--p2p"
        elif quick_conn_pref == "sc":
            command="--sc"
        elif quick_conn_pref == "tor":
            command="--tor"
        else:
            command="--cc"
            country=quick_conn_pref.upper()
        
        command_list = ["protonvpn", "connect", command]
        if country:
            command_list = ["protonvpn", "connect", command, country]
        
        bool_value, result =  self.root_command(command_list)
        return self.get_display_message(bool_value, result)

    def quick_connect(self):
        try:
            secure_core = get_gui_config("connections", "display_secure_core")
        except:
            secure_core = False

        command = ["protonvpn", "connect", ("-f" if secure_core == "False" else "--sc")]
        bool_value, result =  self.root_command(command)
        return self.get_display_message(bool_value, result)

    def last_connect(self):
        command = ["protonvpn", "reconnect"]
        bool_value, result =  self.root_command(command)
        return self.get_display_message(bool_value, result)
    
    def random_connect(self):
        command = ["protonvpn", "connect", "--random"]
        bool_value, result =  self.root_command(command)
        return self.get_display_message(bool_value, result)

    def disconnect(self):
        command = ["protonvpn", "disconnect"]
        bool_value, result =  self.root_command(command)
        return bool_value, self.get_display_message(bool_value, result)

    def check_for_updates(self):
        latest_release = ''
        pip3_installed = False

        try:
            is_pip3_installed = subprocess.run(["pip3", "show", "protonvpn-gui"],stdout=subprocess.PIPE) # nosec
        except:
            return False

        if is_pip3_installed.returncode == 0:
            is_pip3_installed = is_pip3_installed.stdout.decode().split("\n")
            for el in is_pip3_installed:
                if "Location:" in el:
                    el = el.split(" ")[1].split("/")
                    if not ".egg" in el[-1]:
                        pip3_installed = True
                        # print(".egg" in el[-1])
                        break           

        try:
            check_version = requests.get(GITHUB_URL_RELEASE, timeout=2)
            latest_release =  check_version.url.split("/")[-1][1:]
        except (requests.exceptions.ConnectionError,
                requests.exceptions.ConnectTimeout):
            return False

        return (latest_release, pip3_installed)

    def get_display_message(self, bool_value, result):
        display_message = result
        if bool_value:
            server_name = get_server_protocol_from_cli(result)
            display_message = "You are connected to <b>{}</b>!".format(server_name)

        return display_message

    def root_command(self, command_list):
        # inject sudo_type from configurations
        command_list.insert(0, self.sudo_type)

        process = subprocess.Popen(command_list, stdout=subprocess.PIPE, stderr=subprocess.PIPE) # nosec
        timeout = False

        try:
            outs, errs = process.communicate(timeout=self.timeout_value)
        except subprocess.TimeoutExpired:
            timeout = True
            process.kill()
            outs, errs = process.communicate()

        errs = errs.decode().lower()
        outs = outs.decode().lower()
        gui_logger.debug("errs: {}\nouts: {}".format(errs, outs))

        if "terminal is required" in errs:
            return (False, "Privilege escalation is required and PolKit Support is not enabled.\nPlease launch the app either from within a terminal or enable PolKit Support.")

        if "dismissed" in errs and not timeout:
            return (False, "Privilege escalation was dismissed.")
        
        if not "dismissed" in errs and timeout:
            return (False, "Request timed out, either because of insufficient privileges\nor network/api issues.")

        if "authentication failed" in outs:
            return (False, "Authentication failed!\nPlease make sure that your username and password is correct.")

        if "connected" in outs or "disconnected" in outs:
            return (True, outs.upper())

        return (False, errs)

    def diagnose(self):
        reccomendation = '' 
        end_openvpn_process_guide = """\n
        sudo pkill openvpn\n
        or\n
        sudo pkill -9 openvpn
        """
        restore_ip_tables_guide ="""\n
        sudo iptables -F
        sudo iptables -P INPUT ACCEPT
        sudo iptables -P OUTPUT ACCEPT
        sudo iptables -P FORWARD ACCEPT
        """
        restart_netwman_guide = """\n
        sudo systemctl restart NetworkManager
        """

        has_internet, is_killswitch_enabled, is_ovpnprocess_running, is_dns_protection_enabled = self.get_diagnose_settings()
        is_custom_resolv_conf = self.check_custom_dns()
        is_splitunn_enabled = self.check_split_tunneling()
        
        # Reccomendations based on known issues
        if not has_internet:
            if is_ovpnprocess_running:
                reccomendation = reccomendation + "\nYou have no internet connection and a VPN process is running.\n"
                reccomendation = reccomendation + "This might be due to a DNS misconfiguration or lack of internet connection. You can try to disconnecto from the VPN by clicking on \"Disconnect\" or following the instructions below.\n"
                reccomendation = reccomendation + "<b>Warning:</b> By doing this you are ending your VPN process, which might end exposing your traffic upon reconnecting, do at your own risk." + end_openvpn_process_guide
            elif not is_ovpnprocess_running:
                if is_killswitch_enabled:
                    reccomendation = reccomendation + "\nYou Have killswitch enabled, which might be blocking your connection.\nTry to flush and then reconfigure your IP tables."
                    reccomendation = reccomendation + "<b>Warning:</b> By doing this you are clearing all of your killswitch configurations. Do at your own risk." + restore_ip_tables_guide
                elif is_custom_resolv_conf["logical"]:
                    reccomendation = reccomendation + "\nCustom DNS is still present in resolv.conf even though you are not connected to a server. This might be blocking your from establishing a non-encrypted connection.\n"
                    reccomendation = reccomendation + "Try to restart your network manager to restore default configurations:" + restart_netwman_guide
                elif is_custom_resolv_conf["logical"] is None:
                    reccomendation = reccomendation + "\nNo running VPN process was found, though DNS configurations are lacking in resolv.conf.\n"
                    reccomendation = reccomendation + "This might be due to some error or corruption during DNS restoration or lack of internet connection.\n"
                    reccomendation = reccomendation + "Try to restart your network manager to restore default configurations, if it still does not work, then you probably experiencing some internet connection issues." + restart_netwman_guide
                else:
                    reccomendation = "\nYou have no internet connection.\nTry to connect to a different nework to resolve the issue."
            else:
                reccomendation = "<b>Unkown problem!</b>"
        else:
            reccomendation = "\nYour system seems to be ok. There are no reccomendations at the moment."

        return (reccomendation, has_internet, is_custom_resolv_conf, 
                is_killswitch_enabled, is_ovpnprocess_running, 
                is_dns_protection_enabled,is_splitunn_enabled)

    def get_diagnose_settings(self):
        # Check if there is internet connection
            # Depending on next questions, some actions might be suggested.
        has_internet = check_internet_conn(request_bool=True)
        
        # Check if killswitch is enabled
            # Advice to restore IP tables manually and restart netowrk manager.
        is_killswitch_enabled = True if get_config_value("USER", "killswitch") == 1 else False

        # Check if VPN is running
            # If there is a OpenVPN process running in the background, kill it.
        is_ovpnprocess_running = is_connected()

        # Check if custom DNS is enabled
            # If there is no VPN connection and also no internet, then it is a DNS issue.
        is_dns_protection_enabled = False if get_config_value("USER", "dns_leak_protection") == "0" or (not get_config_value("USER", "custom_dns") is None and get_config_value("USER", "dns_leak_protection") == "0") else True

        return has_internet, is_killswitch_enabled, is_ovpnprocess_running, is_dns_protection_enabled

    def check_custom_dns(self):
        """Check if custom DNS is being used. 
        
        It might that the user has disabled the custom DNS settings but the file still resides in system settings.
        """
        is_custom_resolv_conf = {
            "logical": False,
            "display": "Original"
        }

        with open("/etc/resolv.conf") as f:
            lines = f.readlines()

            # remove \n from all elements
            lines = map(lambda l: l.strip(), lines)
            # remove empty elements
            lines = list(filter(None, lines))

            if len(lines) < 2:
                is_custom_resolv_conf["logical"] = None
                is_custom_resolv_conf["display"] = "Missing"
            else:
                for item in lines:
                    if "protonvpn" in item.lower():
                        is_custom_resolv_conf["logical"] = True
                        is_custom_resolv_conf["display"] = "Custom"

        return is_custom_resolv_conf

    def check_split_tunneling(self):
        try:
            is_splitunn_enabled = True if get_config_value("USER", "split_tunnel") == "1" else False
        except (KeyError, IndexError) as e:
            gui_logger.debug(e)
            return False

        return is_splitunn_enabled

    def set_individual_server(self, servername, images_dict, servers, feature):
        server_tiers = {0: "Free", 1: "Basic", 2: "Plus/Visionary"}
        features = {0: "Normal", 1: "Secure-Core", 2: "Tor", 4: "P2P"}

        secure_core = False

        load = str(get_server_value(servername, "Load", servers)).rjust(3, " ")
        load = load + "%"               

        tier = server_tiers[get_server_value(servername, "Tier", servers)]
        
        if not "Plus/Visionary".lower() == tier.lower():
            plus_feature = images_dict["empty_pix"]
        else:
            plus_feature = images_dict["plus_pix"]

        server_feature = features[get_server_value(servername, 'Features', servers)].lower()
        
        if server_feature == "Normal".lower():
            feature = images_dict["empty_pix"]
        elif server_feature == "P2P".lower():
            feature = images_dict["p2p_pix"]
        elif server_feature == "Tor".lower():
            feature = images_dict["tor_pix"]
        else:
            # Should be secure core
            secure_core = True

        return (servername, plus_feature, feature, load, secure_core)

    def get_flag_path(self, country):
        for k,v in country_codes.items():
            if country == v:
                flag_path = SMALL_FLAGS_BASE_PATH+"{}.png".format(v)
                break
            else:
                flag_path = SMALL_FLAGS_BASE_PATH+"Unknown.png"

        return flag_path

    def get_country_servers(self, servers):
        countries = {}
        for server in servers:
            country = get_country_name(server["ExitCountry"])
            if country not in countries.keys():
                countries[country] = []
            countries[country].append(server["Name"])

        country_servers = {} 
        # Order server list by country alphabetically
        countries = collections.OrderedDict(sorted(countries.items()))

        for country in countries:
            country_servers[country] = sorted(countries[country], key=lambda s: get_server_value(s, "Load", servers))

        return country_servers

    def create_features_img(self, GdkPixbuf):
        # Create empty image
        empty_path = FEATURES_BASE_PATH+"normal.png"
        empty_pix = GdkPixbuf.Pixbuf.new_from_file_at_size(empty_path, 15,15)
        # Create P2P image
        p2p_path = FEATURES_BASE_PATH+"p2p-arrows.png"
        p2p_pix = GdkPixbuf.Pixbuf.new_from_file_at_size(p2p_path, 15,15)
        # Create TOR image
        tor_path = FEATURES_BASE_PATH+"tor-onion.png"
        tor_pix = GdkPixbuf.Pixbuf.new_from_file_at_size(tor_path, 15,15)
        # Create Plus image
        plus_server_path = FEATURES_BASE_PATH+"plus-server.png"
        plus_pix = GdkPixbuf.Pixbuf.new_from_file_at_size(plus_server_path, 15,15)

        images_dict = {
            "empty_pix": empty_pix,
            "p2p_pix": p2p_pix,
            "tor_pix": tor_pix,
            "plus_pix": plus_pix,
        }
        return images_dict

    def get_country_avrg_features(self, country, country_servers, servers, only_secure_core):
        """Function that returns average load and features of a specific country.
        """
        features = {0: "Normal", 1: "Secure-Core", 2: "Tor", 4: "P2P"}
        # Variables for average per country
        count = 0
        load_sum = 0
        # Variable for feature per country
        features_per_country = set()

        order_dict = {
            "normal": 0,
            "p2p": 1,
            "tor": 2,
            "secure-core": 3,
        }
        top_choice = 0

        for servername in country_servers[country]:
            # Get average per country
            load_sum = load_sum + int(str(get_server_value(servername, "Load", servers)).rjust(3, " "))
            count += 1
            
            # Get features per country
            feature = features[get_server_value(servername, 'Features', servers)]
            features_per_country.add(feature)
        
        # Convert set to list
        country_feature_list = list(features_per_country)
        
        for feature in country_feature_list:
            for k,v in order_dict.items():
                if feature.lower() == k.lower():
                    # if top_choice < v and (not only_secure_core and not v > 2):
                    if top_choice < v:
                        top_choice = v

        if top_choice == 0:
            top_choice = "normal"
        elif top_choice == 1:
            top_choice = "p2p"
        elif top_choice == 2:
            top_choice = "tor"
        else:
            top_choice = "secure-core"

        # print(country,top_choice)

        return  (str(int(round(load_sum/count)))+"%", top_choice) 

    @property
    def sudo_type(self):
        try:
            is_polkit_enabled =  int(get_gui_config("general_tab", "polkit_enabled"))
        except (KeyError, NameError):
            return "sudo"

        return_val = "sudo"

        if is_polkit_enabled == 1:
            return_val = "pkexec"

        return return_val


import os
import sys
import time
import configparser

# Remote imports
from protonvpn_cli.constants import CONFIG_FILE, CONFIG_DIR, PASSFILE #noqa
from protonvpn_cli.utils import set_config_value, pull_server_data #noqa

# Local imports
from ..gui_logger import gui_logger
from ..constants import (
    TRAY_CFG_SERVERLOAD, 
    TRAY_CFG_SERVENAME, 
    TRAY_CFG_DATA_TX, 
    TRAY_CFG_TIME_CONN
)

class LoginPresenter:
    def __init__(self, login_service, queue):
        self.login_service = login_service
        self.queue = queue

    def on_login(self, **kwargs):
        """Function that initializes a user profile.
        """      
        username_field = kwargs.get("username_field")
        password_field = kwargs.get("password_field")
        
        protonvpn_plans = {
            '1': kwargs.get("member_free_radio"),
            '2': kwargs.get("member_basic_radio"),
            '3': kwargs.get("member_plus_radio"),
            '4': kwargs.get("member_visionary_radio"),
        }

        user_data = self.login_service.prepare_initilizer(username_field, password_field, protonvpn_plans)

        if not self.login_service.intialize_cli_config():
            self.queue.put(dict(action="update_dialog", label="Couldn't create folder for cli configurations."))
            return False 

        if not self.login_service.setup_user(user_data):
            self.queue.put(dict(action="update_dialog", label="Couldn't intialize your profile.\nMake sure to start the GUI from within the terminal if this is your first time."))
            return False

        if not self.login_service.initialize_gui_config():
            self.queue.put(dict(action="update_dialog", label="Couldn't create folder for application configurations."))
            return False
        
        self.queue.put(dict(action="update_dialog", label="Your profile was successfully created."))
        return True
        
        



import os
import time
import datetime
import subprocess

# Remote imports
from protonvpn_cli.utils import (
    get_country_name,
    get_config_value,
    is_connected,
    get_transferred_data,
    pull_server_data,
    get_servers,
    get_server_value
)

import gi
gi.require_version('Gtk', '3.0')
gi.require_version('Notify', '0.7')
gi.require_version('AppIndicator3', '0.1')
from gi.repository import Gtk, GObject, GdkPixbuf
from gi.repository import Notify
from gi.repository import AppIndicator3 as appindicator

# Local imports
from .gui_logger import gui_logger
from .constants import (
    TRAY_CFG_SERVERLOAD, 
    TRAY_CFG_SERVENAME, 
    TRAY_CFG_DATA_TX, 
    TRAY_CFG_TIME_CONN, 
    CURRDIR,
)
from .utils import get_gui_config, set_gui_config

LOGO_PATH = os.path.join(CURRDIR, "resources/img/logo/protonvpn_logo.png")
ALT_LOGO_PATH = os.path.join(CURRDIR, "resources/img/logo/protonvpn_logo_alt.png")

class ProtonVPNIndicator:
    def __init__(self):
        self.gtk = Gtk
        self.gobject = GObject
        self.display_serverload = False
        self.connection_error = False
        self.serverload_msg = "Load: -"
        self.menu = self.menu()
        self.ind = appindicator.Indicator.new(
            "ProtonVPN GUI Indicator", 
            "protonvpn-gui-indicator", 
            appindicator.IndicatorCategory.APPLICATION_STATUS)
        self.ind.set_status(appindicator.IndicatorStatus.ACTIVE)
        self.ind.set_menu(self.menu)

        self.tray_title = "ProtonVPN"
        self.notify = Notify
        self.notify.init("ProtonVPN Tray")

        gui_logger.debug("TRAY >>> Starting tray.")
        # Get first server load
        self.update_serverload(None)
        # Call main loop
        self.main_loop(None)
        
        self.gobject.timeout_add_seconds(5, self.main_loop, None)
        self.gobject.timeout_add_seconds(910, self.update_serverload, None)

        self.gtk.main()

    def menu(self):
        self.menu = self.gtk.Menu()
        
        self.server_load = self.gtk.MenuItem(label='')
        self.menu.append(self.server_load)
        
        self.data_sent = self.gtk.MenuItem(label='')
        self.menu.append(self.data_sent)

        self.data_rec = self.gtk.MenuItem(label='')
        self.menu.append(self.data_rec)

        self.time_conn = self.gtk.MenuItem(label='')
        self.menu.append(self.time_conn) 

        self.ip = self.gtk.MenuItem(label='')
        self.menu.append(self.ip)

        self.separator_0 = self.gtk.SeparatorMenuItem()
        self.menu.append(self.separator_0)
        self.separator_0.show()

        self.q_connect = self.gtk.MenuItem(label='Quick Connect')
        self.q_connect.connect('activate', self.quick_connect)
        self.menu.append(self.q_connect)
        self.q_connect.show()

        self.disconn = self.gtk.MenuItem(label='Disconnect')
        self.disconn.connect('activate', self.disconnect)
        self.menu.append(self.disconn)
        self.disconn.show()

        self.separator_1 = self.gtk.SeparatorMenuItem()
        self.menu.append(self.separator_1)
        self.separator_1.show()

        self.gui = self.gtk.MenuItem(label='Show GUI')
        self.gui.connect('activate', self.show_gui)
        self.menu.append(self.gui) 
        self.gui.show()
        
        self.separator_2 = self.gtk.SeparatorMenuItem()
        self.menu.append(self.separator_2)
        self.separator_2.show()

        self.exittray = self.gtk.MenuItem(label='Quit')
        self.exittray.connect('activate', self.quit_indicator)
        self.menu.append(self.exittray)
        self.exittray.show()

        return self.menu

    def main_loop(self, _):
        """Main loop that updates all labels.
        """
        tray_icon = ALT_LOGO_PATH
        self.display_serverload = False
        display_data_rec = False
        display_server = False
        display_time_conn = False

        if is_connected():
            tray_icon = LOGO_PATH
            settings = self.get_tray_settings()

            self.display_serverload = True if settings["display_serverload"] else False
            display_data_rec = True if settings["display_data_tx"] else False
            display_server = True if settings["display_server"] else False
            display_time_conn = True if settings["display_time_conn"] else False
            
        self.display_extra_info(
                                display_serverload=self.display_serverload,
                                display_data_rec=display_data_rec,
                                display_server=display_server, 
                                display_time_conn=display_time_conn)

        self.ind.set_icon_full(tray_icon, 'protonvpn')

        return True
    
    def update_serverload(self, _):
        """Updates server load.
        """
        gui_logger.debug("TRAY >>> Updating server load in update_serverload.")
        
        connected_server = False
        load = False

        # force_pull servers
        try:
            pull_server_data()
        except:
            gui_logger.debug("[!] Could not pull from servers, possible due to unstable connection.")
            self.connection_error = True
            return True
        
        try:
            connected_server = get_config_value("metadata", "connected_server")
        except (KeyError, IndexError):
            gui_logger.debug("[!] Could not find specified key.")
            return True

        # get_servers
        try:
            servers = get_servers()
        except:
            return True

        # get server load
        try:
            load = get_server_value(connected_server, "Load", servers)
        except (KeyError, IndexError):
            gui_logger.debug("[!] Unable to get server load.")
            return True

        self.serverload_msg = "Load: {}%".format(load)
        
        return True

    def quick_connect(self, _):
        """Makes a quick connection by making a cli call to protonvpn-cli-ng"""
        gui_logger.debug("TRAY >>> Starting quick connect")

        msg = "Administrator access was not given to quick connect"
        timeout = False

        self.notify.Notification.new(self.tray_title, "Starting quick connect...", LOGO_PATH).show()

        process = subprocess.Popen([self.sudo_type, "protonvpn", "connect", "-f"], stdout=subprocess.PIPE, stderr=subprocess.PIPE) # nosec
    
        try:
            outs, errs = process.communicate(timeout=20)
        except subprocess.TimeoutExpired:
            msg = "Unable to connect, make sure that your connection is stable and that the you app has been given administrator access"
            timeout = True
            # process.kill()
            outs, errs = process.communicate()

            
        errs = errs.decode().lower()
        outs = outs.decode().lower()

        if not "dismissed" in errs and not timeout:
            msg = "Unable to connect to VPN"
            if "terminal is required" in errs:
                msg = "Administrator access is needed to connect to VPN. Either enable PolKit Support from within the GUI or launch Tray App from terminal"
            if "connected" in outs:
                msg = "Connected"

        self.notify.Notification.new(self.tray_title, msg, LOGO_PATH).show()
        
        gui_logger.debug("TRAY >>> errs: {}\nouts:{}".format(outs, errs))

    def disconnect(self, _):
        """Disconnects from a current vpn connection"""
        gui_logger.debug("TRAY >>> Starting disconnect.")

        msg = "Administrator access was not given to disconnect"
        timeout = False
        self.notify.Notification.new(self.tray_title, "Disconnecting from VPN...", LOGO_PATH).show()
        
        process = subprocess.Popen([self.sudo_type, "protonvpn", "disconnect"], stdout=subprocess.PIPE, stderr=subprocess.PIPE) # nosec

        try:
            outs, errs = process.communicate(timeout=7)
        except subprocess.TimeoutExpired:
            msg = "Unable to disconnect, make sure that the app has been given administrator access to do so"
            timeout = True
            # process.kill()
            outs, errs = process.communicate()

        errs = errs.decode().lower()
        outs = outs.decode().lower()

        if not "dismissed" in errs and not timeout:
            msg = "Unable to disconnect from VPN"
            if "terminal is required" in errs:
                msg = "Administrator access is needed to disconnect. Either enable PolKit Support from within the GUI or launch Tray App from within a terminal"
            if "disconnected" in outs:
                msg = "Disconnected"

        self.notify.Notification.new(self.tray_title, msg, LOGO_PATH).show()
        
        gui_logger.debug("TRAY >>> errs: {}\nouts:{}".format(outs, errs))

    def show_gui(self, _):
        """Displays the GUI."""        
        gui_logger.debug("TRAY >>> Starting to display GUI.")

        self.notify.Notification.new(self.tray_title, "Displaying ProtonVPN GUI", LOGO_PATH).show()

        process = subprocess.Popen(["protonvpn-gui"], stdout=subprocess.PIPE, stderr=subprocess.PIPE) # nosec
        outs, errs = process.communicate()
      
        gui_logger.debug("TRAY >>> errs: {}\nouts:{}".format(outs, errs))

    def get_tray_settings(self):
        """Gets and returns tray settings from config file.
        Returns: dict
            - Dictionary with boolean values for each display configuration.
        """

        resp_dict = {
            "display_serverload": False,
            "display_data_tx": False,
            "display_server": False,
            "display_time_conn": False,
        }

        try: 
            resp_dict["display_serverload"] = int(get_gui_config("tray_tab", TRAY_CFG_SERVERLOAD))
        except (KeyError, IndexError):
            gui_logger.debug("[!] Could not find display_serverload in config file.")
        
        try: 
            resp_dict["display_server"] = int(get_gui_config("tray_tab", TRAY_CFG_SERVENAME))
        except (KeyError, IndexError):
            gui_logger.debug("[!] Could not find display_server in config file.")

        try: 
            resp_dict["display_data_tx"] = int(get_gui_config("tray_tab", TRAY_CFG_DATA_TX))
        except (KeyError, IndexError):
            gui_logger.debug("[!] Could not find display_data_tx in config file.")
        
        try: 
            resp_dict["display_time_conn"] = int(get_gui_config("tray_tab", TRAY_CFG_TIME_CONN))
        except (KeyError, IndexError):
            gui_logger.debug("[!] Could not find display_time_conn in config file.")

        return resp_dict

    def display_extra_info(self, **kwrgs):
        """Either displays or hides information based on tray display configurations.
        """

        if kwrgs["display_serverload"]: 
            self.server_load.get_child().set_text(self.serverload_msg)
            self.server_load.show()
        else:
            self.server_load.get_child().set_text("Load: -")
            self.server_load.hide()

        if kwrgs["display_server"]: 
            server = get_config_value("metadata", "connected_server")
            self.ind.set_label(server, "")
        else:
            self.ind.set_label("", "")

        if kwrgs["display_data_rec"]:
            received, sent = self.data_sent_received()

            display_data_rec = "Received: {}".format(received)
            display_data_sent = "Sent: {}".format(sent)

            self.data_rec.get_child().set_text(display_data_rec)
            self.data_sent.get_child().set_text(display_data_sent)

            self.data_rec.show()
            self.data_sent.show()
        else: 
            self.data_rec.hide()
            self.data_sent.hide()

        if kwrgs["display_time_conn"]:
            display_time_conn = self.time_connected()

            self.time_conn.get_child().set_text("Connection time: {}".format(display_time_conn))

            self.time_conn.show()
        else: 
            self.time_conn.hide()

    def data_sent_received(self):
        """Get and returns ammount of sent and received data.
        """

        sent_amount, received_amount = get_transferred_data()

        sent_amount = sent_amount if is_connected else ""
        received_amount = received_amount if is_connected else ""

        return (received_amount, sent_amount)

    def time_connected(self):
        """Gets and returns the connection time length.
        """

        try:
            connected_time = get_config_value("metadata", "connected_time")
            connection_time = time.time() - int(connected_time)
            connection_time = str(datetime.timedelta(seconds=connection_time)).split(".")[0]
        except (KeyError, IndexError):
            connection_time = False
    
        connection_time = connection_time if connection_time else ""

        return connection_time

    @property
    def sudo_type(self):
        try:
            is_polkit_enabled =  int(get_gui_config("general_tab", "polkit_enabled"))
        except (KeyError, NameError):
            return "sudo"

        return_val = "sudo"

        if is_polkit_enabled == 1:
            return_val = "pkexec"

        return return_val 

    def quit_indicator(self, _):
        """Quit/Stop the tray icon.
        """
        self.gtk.main_quit()



import os
import sys
import time
import shutil
import subprocess
import configparser

# Remote imports
from protonvpn_cli.constants import CONFIG_FILE, CONFIG_DIR, PASSFILE #noqa
from protonvpn_cli.utils import set_config_value, pull_server_data #noqa
from protonvpn_cli.logger import logger

# Local imports
from ..gui_logger import gui_logger
from ..constants import (
    TRAY_CFG_SERVERLOAD, 
    TRAY_CFG_SERVENAME, 
    TRAY_CFG_DATA_TX, 
    TRAY_CFG_TIME_CONN, 
    GUI_CONFIG_FILE,
    GUI_CONFIG_DIR,
    TRAY_CFG_SUDO
)
from ..utils import is_polkit_installed

class LoginService:

    def prepare_initilizer(self, username_field, password_field, protonvpn_plans):
        """Funciton that collects and prepares user input from login window.
        Returns:
        ----
        - A dictionary with username, password, plan type and default protocol.
        """
        # Get user specified protocol
        protonvpn_plan = ''
                
        # Get user plan
        for k,v in protonvpn_plans.items():
            if v:
                protonvpn_plan = k
                break
        
        user_data = {
            'username': username_field,
            'password': password_field,
            'protonvpn_plan': int(protonvpn_plan),
            'openvpn_protocol': "tcp"
        }
        return user_data
        
    def setup_user(self, user_data):
        ovpn_username = user_data['username']
        ovpn_password = user_data['password']
        user_tier = user_data['protonvpn_plan']
        user_protocol = user_data['openvpn_protocol']

        if not self.generate_user_pass_file(ovpn_username, ovpn_password):
            shutil.rmtree(CONFIG_DIR)
            return False

        gui_logger.debug("Passfile created")
        
        pull_server_data(force=True)

        if user_tier == 4:
            user_tier = 3
        user_tier -= 1

        set_config_value("USER", "username", ovpn_username)
        set_config_value("USER", "tier", user_tier)
        set_config_value("USER", "default_protocol", user_protocol)
        set_config_value("USER", "dns_leak_protection", 1)
        set_config_value("USER", "custom_dns", None)
        set_config_value("USER", "killswitch", 0)
        set_config_value("USER", "split_tunnel", 0)

        set_config_value("USER", "initialized", 1)

        return True

    def intialize_cli_config(self):
        if not os.path.isdir(CONFIG_DIR):
            gui_logger.debug("CLI config folder created.")
            os.mkdir(CONFIG_DIR)

        config = configparser.ConfigParser()
        config["USER"] = {
            "username": "None",
            "tier": "None",
            "default_protocol": "None",
            "initialized": "0",
            "dns_leak_protection": "1",
            "custom_dns": "None",
            "check_update_interval": "3",
            "killswitch": "0",
            "split_tunnel": "0",
            "api_domain": "https://api.protonvpn.ch"
        }
        config["metadata"] = {
            "last_api_pull": "0",
            "last_update_check": str(int(time.time())),
        }

        try:
            with open(CONFIG_FILE, "w") as f:
                config.write(f)

            gui_logger.debug("pvpn-cli.cfg initialized")
        except:
            shutil.rmtree(CONFIG_DIR)
            return False

        return True

    def initialize_gui_config(self):
        if not os.path.isdir(GUI_CONFIG_DIR):
            gui_logger.debug("GUI config folder created.")
            os.mkdir(GUI_CONFIG_DIR)

        gui_config = configparser.ConfigParser()
        gui_config["connections"] = {
            "display_secure_core": False
        }
        gui_config["general_tab"] = {
            "start_min": False,
            "start_on_boot": False,
            "show_notifications": False,
            "polkit_enabled": 0
        }
        gui_config["tray_tab"] = {
            TRAY_CFG_DATA_TX: "0",
            TRAY_CFG_SERVENAME: "0",
            TRAY_CFG_TIME_CONN: "0",
            TRAY_CFG_SERVERLOAD: "0",
        }
        gui_config["conn_tab"] = {
            "autoconnect": "dis",
            "quick_connect": "dis",
        }

        with open(GUI_CONFIG_FILE, "w") as f:
            gui_config.write(f)
            gui_logger.debug("pvpn-gui.cfg initialized.")

        if not os.path.isfile(GUI_CONFIG_FILE):
            gui_logger.debug("File not found: pvpn-gui.cfg")
            shutil.rmtree(CONFIG_DIR)
            shutil.rmtree(GUI_CONFIG_DIR)
            return False

        return True

    def generate_user_pass_file(self, username, password):
        user_pass = "'{}\n{}'".format(username, password)
        echo_to_passfile = "echo -e {} > {}".format(user_pass, PASSFILE)

        sudo_type = "pkexec" if is_polkit_installed else "sudo"

        try:
            output = subprocess.check_output([sudo_type, "bash", "-c", echo_to_passfile], stderr=subprocess.STDOUT, timeout=8)
            set_config_value("USER", "username", username)
        except (subprocess.TimeoutExpired, subprocess.CalledProcessError) as e:
            gui_logger.debug(e)
            return False   

        gui_logger.debug("Passfile generated")
        return True


import re
import os
import time
import datetime
import requests
import subprocess
import collections
import configparser

from protonvpn_cli.utils import (
    pull_server_data,
    get_servers,
    get_country_name,
    get_server_value,
    get_config_value,
    is_connected,
    get_transferred_data,
)
from protonvpn_cli.country_codes import country_codes
from protonvpn_cli.constants import SPLIT_TUNNEL_FILE, USER

from .constants import (
    PATH_AUTOCONNECT_SERVICE, 
    TEMPLATE, 
    VERSION, 
    SERVICE_NAME,  
    TRAY_CFG_DICT,
    TRAY_CFG_DATA_TX,
    TRAY_CFG_SERVENAME,
    TRAY_CFG_SERVERLOAD,
    TRAY_CFG_TIME_CONN,
    GUI_CONFIG_FILE,
    LARGE_FLAGS_BASE_PATH,
    SMALL_FLAGS_BASE_PATH,
    FEATURES_BASE_PATH
)

from .gui_logger import gui_logger

def tab_style_manager(tab_to_show: str, tab_dict):
    for k, v in tab_dict.items():
        if k == tab_to_show:
            v.add_class("active_tab")
            v.remove_class("inactive_tab")
        else:
            v.add_class("inactive_tab")
            v.remove_class("active_tab")

def get_gui_config(group, key):
    """Return specific value from GUI_CONFIG_FILE as string"""
    config = configparser.ConfigParser()
    config.read(GUI_CONFIG_FILE)

    return config[group][key]

def set_gui_config(group, key, value):
    """Write a specific value to GUI_CONFIG_FILE"""

    config = configparser.ConfigParser()
    config.read(GUI_CONFIG_FILE)
    config[group][key] = str(value)

    gui_logger.debug(
        "Writing {0} on {1} to [{2}] in gui config file".format(str(value), key, group)
    )

    with open(GUI_CONFIG_FILE, "w+") as f:
        config.write(f)

def initialize_gui_config():
        gui_config = configparser.ConfigParser()
        gui_config["connections"] = {
            "display_secure_core": False
        }
        gui_config["general_tab"] = {
            "start_min": False,
            "start_on_boot": False,
            "show_notifications": False,
        }
        gui_config["tray_tab"] = {
            TRAY_CFG_DATA_TX: "0",
            TRAY_CFG_SERVENAME: "0",
            TRAY_CFG_TIME_CONN: "0",
            TRAY_CFG_SERVERLOAD: "0",
        }
        gui_config["conn_tab"] = {
            "autoconnect": "dis",
            "quick_connect": "dis",
        }

        with open(GUI_CONFIG_FILE, "w") as f:
            gui_config.write(f)
            gui_logger.debug("pvpn-gui.cfg initialized.")

        if not os.path.isfile(GUI_CONFIG_FILE):
            print("something")
            gui_logger.debug("Unablt to initialize pvpn-gui.cfg. {}".format(Exception))
            return False

        return True

def get_server_protocol_from_cli(raw_result, return_protocol=False):
    """Function that collects servername and protocol from CLI print statement after establishing connection.
    """
    if type(raw_result) is not bool:
        display_message = raw_result.split("\n")
        display_message = display_message[-3:]
        
        server_name = [re.search("[A-Z-a-z]{1,7}#[0-9]{1,4}", text) for text in display_message]

        if any(server_name):
            if return_protocol:
                protocol = re.search("(UDP|TCP)", display_message[0])
                return (server_name[0].group(), protocol.group())
            return server_name[0].group()

    return False

def check_internet_conn(request_bool=False):
    """Function that checks for internet connection.
    """
    gui_logger.debug(">>> Running \"check_internet_conn\".")

    return custom_call_api(request_bool=request_bool)

def custom_call_api(endpoint=False, request_bool=False):
    """Function that is a custom call_api with a timeout of 6 seconds. This is mostly used to check for API access and also for internet access.
    """
    api_domain = "https://api.protonvpn.ch"
    if not endpoint:
        endpoint = "/vpn/location"

    url = api_domain + endpoint

    headers = {
        "x-pm-appversion": "Other",
        "x-pm-apiversion": "3",
        "Accept": "application/vnd.protonmail.v1+json"
    }

    gui_logger.debug("Initiating custom API Call: {0}".format(url))

    try:
        response = requests.get(url, headers=headers, timeout=6)
    except (requests.exceptions.ConnectionError,
            requests.exceptions.ConnectTimeout,
            requests.exceptions.ReadTimeout):
        gui_logger.debug("Error connecting to ProtonVPN API. Connection either timed out or were unable to connect.")
        return False

    try:
        response.raise_for_status()
    except requests.exceptions.HTTPError:
        gui_logger.debug("Bad Return Code: {0}".format(response.status_code))
        return False

    if request_bool:
        return True

    return response.json()

def find_cli():
    """Function that searches for the CLI. Returns CLIs path if it is found, otherwise it returns False.
    """
    protonvpn_path = subprocess.run(['which', 'protonvpn'], stdout=subprocess.PIPE, stderr=subprocess.PIPE) # nosec
    if protonvpn_path.returncode == 1:
        gui_logger.debug("[!] Unable to run \"find protonvpn-cli-ng\" subprocess. Result: \"{}\"".format(protonvpn_path))
        protonvpn_path = False

    return protonvpn_path.stdout.decode()[:-1] if (protonvpn_path and protonvpn_path.returncode == 0) else False
       
def custom_get_ip_info():
    """Custom get_ip_info that also returns the country.
    """
    gui_logger.debug("Getting IP Information")
    ip_info = custom_call_api(endpoint="/vpn/location")

    if not ip_info:
        return False
        
    ip = ip_info["IP"]
    isp = ip_info["ISP"]
    country = ip_info["Country"]

    return ip, isp, country

def kill_duplicate_gui_process():
    """Function to kill duplicate/existing protonvpn-linux-gui processes.
    """
    return_message = {
        "message": "Unable to automatically end service. Please manually end the process.",
        "success": False
    }
    
    process = get_gui_processes()
    if len(process) > 1:
        gui_logger.debug("[!] Found following processes: {0}. Will attempt to end \"{1}\"".format(process, process[0]))

        # select first(longest living) process from list
        process_to_kill = process[0]

        timer_start = time.time()

        while len(get_gui_processes()) > 1:
            if time.time() - timer_start <= 10:
                subprocess.run(["kill", process_to_kill]) # nosec
                time.sleep(0.2)
            else:
                subprocess.run(["kill", "-9", process_to_kill]) # nosec
                gui_logger.debug("[!] Unable to pkill process \"{0}\". Will attempt a SIGKILL.".format(process[0]))
                break

        if len(get_gui_processes()) == 1:
            return_message['message'] = "Previous process ended, resuming actual session."        
            return_message['success'] = True
            gui_logger.debug("[!] Process \"{0}\" was ended.".format(process[0]))

    elif len(process) == 1:
        return_message['message'] = "Only one process, normal startup."        
        return_message['success'] = True
        gui_logger.debug(">>> Only one process was found, continuing with normal startup.")

    return return_message

def get_gui_processes():
    """Function that returns all possible running GUI processes. 
    """
    gui_logger.debug(">>> Running \"get_gui_processes\".")

    processes = subprocess.run(["pgrep", "protonvpn-gui"],stdout=subprocess.PIPE) # nosec
    
    processes = list(filter(None, processes.stdout.decode().split("\n"))) 

    gui_logger.debug(">>> Existing process running: {0}".format(processes))

    return processes

def is_polkit_installed():
    """Checks for polkit/pkexec and sets to it if found.
    """
    process = subprocess.run(["which", "pkexec"], stdout=subprocess.PIPE, stderr=subprocess.PIPE) # nosec

    return_response = False
    if process.returncode == 0 and process.stdout.decode().strip("\n").split("/")[-1:][0] == "pkexec":
        return_response = True

    gui_logger.debug(">>> {}.".format(process))

    return return_response



import time
import datetime
import requests
import subprocess
import collections
import concurrent.futures

# Remote imports
from protonvpn_cli.utils import (
    get_config_value, 
    is_connected, 
    get_servers, 
    get_server_value, 
    get_transferred_data, 
    pull_server_data,
    get_country_name
)
from protonvpn_cli.country_codes import country_codes #noqa

# PyGObject import
import gi
gi.require_version('Gtk', '3.0')
from gi.repository import GObject as gobject, GdkPixbuf

# Local imports
from ..gui_logger import gui_logger
from ..constants import GITHUB_URL_RELEASE, VERSION, LARGE_FLAGS_BASE_PATH, SMALL_FLAGS_BASE_PATH, FEATURES_BASE_PATH
from ..utils import (
    get_server_protocol_from_cli,
    get_gui_config,
    set_gui_config,
    check_internet_conn,
    custom_get_ip_info,
)

class DashboardPresenter:
    def __init__(self, dashboard_service, queue):
        self.dashboard_service = dashboard_service
        self.queue = queue

    def on_load(self, objects_dict):
        """Calls load_on_start, which returns False if there is no internet connection, otherwise populates dashboard labels and server list
        """
        display_message = "Could not load necessary resources, there might be connectivity issues."
        time.sleep(2)
        self.queue.put(dict(action="hide_spinner"))
        
        conn = custom_get_ip_info()

        if conn and not conn is None:
            self.on_load_dashboard_content(objects_dict, conn)
        else:
            self.queue.put(dict(action="update_dialog", label=display_message, spinner=False))

    def on_load_dashboard_content(self, objects_dict, conn):
        # self.queue.put(dict(action="update_dialog", label="Populating dashboard...", hide_close_button=True))
        self.queue.put(dict(action="hide_dialog"))

        # Sets Secure-Core switch state
        self.on_load_set_secure_core(objects_dict["secure_core"]["secure_core_switch"], objects_dict["secure_core"]["secure_core_label_style"])

        # Loads the labels
        self.on_update_labels(objects_dict["connection_labels"])
        
        # Loads the server list
        self.on_update_server_list(objects_dict["server_tree_list"]["tree_object"])

    def on_load_set_secure_core(self, secure_core_switch, secure_core_label):
        """Sets Secure-Core switch based on user setting.
        """
        try:
            display_secure_core = get_gui_config("connections", "display_secure_core")
        except KeyError:
            display_secure_core = "False"

        secure_core_label_style = secure_core_label.get_style_context() 

        if display_secure_core == "True":
            secure_core_switch.set_state(True)
            secure_core_label_style.remove_class("disabled_label")
        else:
            secure_core_switch.set_state(False)

    def reload_secure_core_servers(self, **kwargs):
        """Function that reloads server list to either secure-core or non-secure-core.
        """  
        # Sleep is needed because it takes a second to update the information,
        # which makes the button "lag". Temporary solution.
        time.sleep(1)
        return_val = False
        display_message = "Unable to reload servers!"

        if self.dashboard_service.set_display_secure_core(kwargs.get("update_to")):
            return_val = True

            self.on_update_server_list(kwargs.get("tree_object"))
            display_message = "Displaying <b>{}</b> servers!".format("secure-core" if kwargs.get("update_to") == "True" else "non secure-core")

        self.queue.put(dict(action="update_dialog", label=display_message))

    def on_connect_user_selected(self, **kwargs):
        """Function that either connects by selected server or selected country.
        """     
        user_selected_server = kwargs.get("user_selected_server")

        # Check if it should connect to country or server
        if "#" in user_selected_server:
            display_message = self.dashboard_service.connect_to_server(user_selected_server)
        else:
            display_message = self.dashboard_service.connect_to_country(user_selected_server)

        self.queue.put(dict(action="update_dialog", label=display_message))

        self.on_update_labels(kwargs.get("connection_labels"))

    def quick_connect(self, **kwargs):
        """Function that connects to the quickest server.
        """      
        profile_quick_connect = False
        if "profile_quick_connect" in kwargs:
            profile_quick_connect = True

        display_message = self.dashboard_service.quick_connect_manager(profile_quick_connect)
        
        self.queue.put(dict(action="update_dialog", label=display_message))
        
        self.on_update_labels(kwargs.get("connection_labels"))

    def on_last_connect(self, **kwargs):
        """Function that connects to the last connected server.
        """        
        display_message = self.dashboard_service.last_connect()

        self.queue.put(dict(action="update_dialog", label=display_message))

        self.on_update_labels(kwargs.get("connection_labels"))

    def random_connect(self, **kwargs):
        """Function that connects to a random server.
        """
        display_message = self.dashboard_service.random_connect()

        self.queue.put(dict(action="update_dialog", label=display_message))

        self.on_update_labels(kwargs.get("connection_labels"))

    def on_refresh_servers(self, **kwargs):
        """Function that reloads server list to either secure-core or non-secure-core.
        """  
        # Sleep is needed because it takes a second to update the information,
        # which makes the button "lag". Temporary solution.
        time.sleep(1)
        return_val = False

        self.queue.put(dict(action="hide_spinner"))

        conn = custom_get_ip_info()
        if conn and not conn is None:
            self.on_update_server_list(kwargs.get("tree_object"))
            self.queue.put(dict(action="hide_dialog"))
            return_val = True
        else:
            self.queue.put(dict(action="update_dialog", label="Could not update servers!", spinner=False))
        
    def on_disconnect(self, **kwargs):
        """Function that disconnects from the VPN.
        """
        is_disconnected, display_message = self.dashboard_service.disconnect()
        if is_disconnected:
            display_message = "Disconnected from VPN!"
            self.on_update_labels(kwargs.get("connection_labels"), disconnect=True)

        self.queue.put(dict(action="update_dialog", label=display_message))

    def on_check_for_updates(self):
        """Function that searches for existing updates by checking the latest releases on github.
        """
        return_string = "Developer Mode."
        return_val = False
        
        try:
            latest_release, pip3_installed = self.dashboard_service.check_for_updates()
        except:
            latest_release = False

        if not latest_release:  
            return_string = "Failed to check for updates."
        else:
            if latest_release == VERSION:
                return_string = "You have the latest version!"
                return_val = True

            if VERSION < latest_release:
                return_string = "There is a newer release, you should upgrade to <b>v{0}</b>.\n\n".format(latest_release)
                if pip3_installed:
                    return_string = return_string + "You can upgrade with the following command:\n\n<b>sudo pip3 install protonvpn-linux-gui-calexandru2018 --upgrade</b>\n\n"
                else:
                    return_string = return_string + "You can upgrade by <b>first removing this version</b>, and then cloning the new one with the following commands:\n\n<b>git clone https://github.com/calexandru2018/protonvpn-linux-gui</b>\n\n<b>cd protonvpn-linux-gui</b>\n\n<b>sudo python3 setup.py install</b>"
                return_val = True

        self.queue.put(dict(action="update_dialog", label=return_string))

        return return_val

    def on_diagnose(self):
        """Multipurpose message dialog function.
        """
        reccomendation, has_internet, is_custom_resolv_conf, is_killswitch_enabled, is_ovpnprocess_running, is_dns_protection_enabled,is_splitunn_enabled = self.dashboard_service.diagnose()

        result = """
        Has internet:\t\t\t\t<b>{has_internet}</b>
        resolv.conf status:\t\t\t<b>{resolv_conf_status}</b>
        Killswitch enabled:\t\t\t<b>{is_ks_enabled}</b>
        VPN Process Running:\t\t<b>{is_vpnprocess_running}</b>
        DNS Protection Enabled:\t\t<b>{is_dns_enabled}</b>
        Split Tunneling Enabled:\t\t<b>{is_sp_enabled}</b>
        """.format(
            has_internet= "Yes" if has_internet else "No",
            resolv_conf_status=is_custom_resolv_conf["display"],
            is_ks_enabled= "Yes" if is_killswitch_enabled else "No",
            is_vpnprocess_running= "Yes" if is_ovpnprocess_running else "No", 
            is_dns_enabled= "Yes" if is_dns_protection_enabled else "No",
            is_sp_enabled= "Yes" if is_splitunn_enabled else "No")

        gui_logger.debug(result)

        self.queue.put(dict(action="update_dialog", label=result, sub_label="<b><u>Reccomendation:</u></b>\n<span>{recc}</span>".format(recc=reccomendation)))

    def on_update_labels(self, connection_labels_dict, disconnect=False):
        # Update labels
        gobject.idle_add(self.update_labels_status, connection_labels_dict, disconnect)

    def on_update_server_list(self, tree_object):
        # Populate server list
        gobject.idle_add(self.populate_server_list, tree_object)

    def update_labels_status(self, connection_labels_dict, disconnecting):
        """Function prepares data to update labels.
        """
        gui_logger.debug(">>> Running \"update_labels_status\" getting servers, is_connected and connected_server.")
        
        servers = get_servers()

        is_vpn_connected = True if is_connected() else False
        country_cc = False
        load = False

        time_connected_label =      connection_labels_dict[0]["time_connected_label"]
        protocol_label =            connection_labels_dict[0]["protocol_label"]
        conn_disc_button_label =    connection_labels_dict[0]["conn_disc_button_label"]
        ip_label =                  connection_labels_dict[0]["ip_label"]
        server_load_label =         connection_labels_dict[0]["server_load_label"]
        country_label =             connection_labels_dict[0]["country_label"]
        isp_label    =              connection_labels_dict[0]["isp_label"]
        data_received_label =       connection_labels_dict[0]["data_received_label"]
        data_sent_label =           connection_labels_dict[0]["data_sent_label"]
        background_large_flag =     connection_labels_dict[0]["background_large_flag"]
        protonvpn_sign_green =      connection_labels_dict[0]["protonvpn_sign_green"]
        
        try:
            connected_server = get_config_value("metadata", "connected_server")
        except (KeyError, IndexError):
            connected_server = False

        # Get and set server load label
        try:
            load = get_server_value(connected_server, "Load", servers)
        except (KeyError, IndexError):
            gui_logger.debug("[!] Could not find \"server load\" information.")
            
        load = "{0}% Load".format(load) if load and is_vpn_connected else ""
        server_load_label.set_markup('<span>{0}</span>'.format(load))

        # Get and set IP labels. Get also country and ISP
        result = custom_get_ip_info()
        if result:
            ip, isp, country = result
        else:
            ip = "None"
            isp = "None" 
            country = "None"

        country = country.lower()

        for k,v in country_codes.items():
            if (k.lower() == country) or (k.lower() == "uk" and country == "gb"):
                if is_vpn_connected:
                    if k.lower() == "uk" and country == "gb":
                        k = "gb"
                    flag_path = LARGE_FLAGS_BASE_PATH+"{}.jpg".format(k.lower())
                    background_large_flag.set_from_file(flag_path)
                country_cc = v
                break

        protonvpn_sign_green.hide()
        country_server = country_cc if type(country_cc) is not bool else connected_server

        if is_vpn_connected:
            try:
                country_server = country_server + " >> " + connected_server
            except TypeError: 
                country_server = country_server + " >> "

            protonvpn_sign_green.show()
        ip_label.set_markup(ip)
        isp_label.set_markup(isp)

        # Get and set server name
        connected_server = connected_server if connected_server and is_vpn_connected else ""
        country_label.set_markup(country_server if country_server else "")

        # Update sent and received data
        gobject.timeout_add_seconds(1, self.update_sent_received_data, {"is_vpn_connected": is_vpn_connected, "received_label": data_received_label, "sent_label": data_sent_label})
        
        # Check and set VPN status label. Get also protocol status if vpn is connected
        protocol = "No VPN Connection"
        conn_disc_button = "Quick Connect"
        if is_vpn_connected and not disconnecting:
            try:
                connected_to_protocol = get_config_value("metadata", "connected_proto")
                protocol = '<span>OpenVPN >> {0}</span>'.format(connected_to_protocol.upper())
            except (KeyError, IndexError):
                pass
            conn_disc_button = "Disconnect"
        conn_disc_button_label.set_markup(conn_disc_button)

        # Check and set DNS status label
        dns_enabled = get_config_value("USER", "dns_leak_protection")

        # Update time connected label
        gobject.timeout_add_seconds(1, self.update_connection_time, {"is_vpn_connected":is_vpn_connected, "label":time_connected_label})

        # Check and set protocol label
        protocol_label.set_markup(protocol)

    def update_sent_received_data(self, dict_labels):
        tx_amount, rx_amount = get_transferred_data()

        rx_amount = rx_amount if dict_labels["is_vpn_connected"] else ""
        
        dict_labels["received_label"].set_markup('<span>{0}</span>'.format(rx_amount))

        # Get and set sent data
        tx_amount = tx_amount if dict_labels["is_vpn_connected"] else ""
        dict_labels["sent_label"].set_markup('<span>{0}</span>'.format(tx_amount))
        
        return True

    def update_connection_time(self, dict_data):
        connection_time = False
        
        if dict_data["is_vpn_connected"]:
            try:
                connected_time = get_config_value("metadata", "connected_time")
                connection_time = time.time() - int(connected_time)
                connection_time = str(datetime.timedelta(seconds=connection_time)).split(".")[0]
            except (KeyError, IndexError):
                connection_time = False
        
        connection_time = connection_time if connection_time else ""
        dict_data["label"].set_markup('<span>{0}</span>'.format(connection_time))

        return True

    def populate_server_list(self, tree_object):
        """Function that updates server list.
        """
        pull_server_data()

        servers = get_servers()

        only_secure_core = True if get_gui_config("connections", "display_secure_core") == "True" else False

        if servers:
            tree_object.clear()

            country_servers = self.dashboard_service.get_country_servers(servers)
            images_dict = self.dashboard_service.create_features_img(GdkPixbuf)

            for country in country_servers:
                # Get average load and highest feature
                avrg_load, country_feature = self.dashboard_service.get_country_avrg_features(country, country_servers, servers, only_secure_core)

                flag = GdkPixbuf.Pixbuf.new_from_file_at_size(self.dashboard_service.get_flag_path(country), 15,15)
                
                # Check plus servers
                if country_feature == "normal" or country_feature == "p2p":
                    plus_feature = images_dict["empty_pix"]
                else:
                    plus_feature = images_dict["plus_pix"]

                # Check correct feature
                if country_feature == "normal" or country_feature == "secure-core":
                    feature = images_dict["empty_pix"]
                elif country_feature == "p2p":
                    feature = images_dict["p2p_pix"]
                elif country_feature == "tor":
                    feature = images_dict["tor_pix"]

                if country_feature == "secure-core" and only_secure_core:
                    country_row = tree_object.append(None, [flag, country, plus_feature, feature, avrg_load])
                elif not only_secure_core:
                    country_row = tree_object.append(None, [flag, country, plus_feature, feature, avrg_load])

                for servername in country_servers[country]:
                    servername, plus_feature, feature, load, secure_core  = self.dashboard_service.set_individual_server(servername, images_dict, servers, feature)

                    if secure_core and only_secure_core:
                        tree_object.append(country_row, [images_dict["empty_pix"], servername, plus_feature, feature, load])
                    elif not secure_core and not only_secure_core:
                        tree_object.append(country_row, [images_dict["empty_pix"], servername, plus_feature, feature, load])

       



from threading import Thread

from ..gui_logger import gui_logger
from ..constants import (
    UI_DIALOG, 
)

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import GObject as gobject

class DialogView: 
    def __init__(self, interface, Gtk, queue):
        interface.add_from_file(UI_DIALOG)
        interface.connect_signals({
            "close_message_dialog": self.close_message_dialog,
            "MessageDialog_delete_event": self.MessageDialog_delete_event,
        })

        self.messagedialog_window = interface.get_object("MessageDialog")
        self.messagedialog_label = interface.get_object("message_dialog_label")
        self.messagedialog_spinner = interface.get_object("message_dialog_spinner")
        self.messagedialog_sub_label = interface.get_object("message_dialog_sub_label")
        self.messagedialog_sub_label.hide()

        self.message_dialog_close_button = interface.get_object("message_dialog_close_button")

        self.interface = interface 
        self.gtk = Gtk
        self.queue = queue

        thread = Thread(target=self.listener)
        thread.daemon = True
        thread.start()

    def listener(self):
        while True:
            kwargs = self.queue.get()

            try:
                if "display_dialog" in kwargs.get("action"):
                    # self.display_dialog(**kwargs)
                    gobject.idle_add(self.display_dialog, kwargs)
                    self.queue.task_done()
                if "update_dialog" in kwargs.get("action"):
                    # self.update_dialog(**kwargs)
                    gobject.idle_add(self.update_dialog, kwargs)
                    self.queue.task_done()

                if "hide_dialog" in kwargs.get("action"):
                    # self.hide_dialog()
                    gobject.idle_add(self.hide_dialog)
                    self.queue.task_done()
                    
                if "hide_spinner" in kwargs.get("action"):
                    # self.hide_spinner()
                    gobject.idle_add(self.hide_spinner)
                    self.queue.task_done()
            except TypeError:
                gui_logger.debug(">>> Error occurs due to testing.") 

    def display_dialog(self, kwargs):
        self.message_dialog_close_button.show()
        self.messagedialog_sub_label.hide()
        self.messagedialog_spinner.hide()

        if "label" in kwargs:
            self.messagedialog_label.set_markup(kwargs.get("label")) 

        if "spinner" in kwargs and kwargs.get("spinner"):
            self.messagedialog_spinner.show()

        if "sub_label" in kwargs and kwargs.get("sub_label"):
            self.messagedialog_sub_label.set_markup(kwargs.get("sub_label"))
            self.messagedialog_sub_label.show()

        if "hide_close_button" in kwargs and kwargs.get("hide_close_button"):
            self.message_dialog_close_button.hide()
            self.messagedialog_window.connect("destroy", self.gtk.main_quit)
        
        self.messagedialog_window.show()

    def update_dialog(self, kwargs):
        self.message_dialog_close_button.show()
        self.messagedialog_sub_label.hide()
        self.messagedialog_spinner.hide()

        if "label" in kwargs:
            self.messagedialog_label.set_markup(kwargs.get("label")) 

        if "spinner" in kwargs and kwargs.get("spinner"):
            self.messagedialog_spinner.show()

        if "sub_label" in kwargs and kwargs.get("sub_label"):
            self.messagedialog_sub_label.set_markup(kwargs.get("sub_label"))
            self.messagedialog_sub_label.show()

        if "hide_close_button" in kwargs and kwargs.get("hide_close_button"):
            self.message_dialog_close_button.hide()
            self.messagedialog_window.connect("destroy", self.gtk.main_quit)            

    def hide_spinner(self):
        self.messagedialog_spinner.hide()

    def hide_dialog(self):
        self.messagedialog_window.hide()

    def close_message_dialog(self, button):
        """Button/Event handler to close message dialog.
        """
        self.interface.get_object("MessageDialog").hide()
        
    # To avoid getting the MessageDialog destroyed and not being re-rendered again
    def MessageDialog_delete_event(self, window, event):
        """On Delete handler is used to hide the dialog and so that it successfully renders next time it is called
        
        -Returns:Boolean
        - It needs to return True, otherwise the content will not re-render after closing the window
        """
        if window.get_property("visible") is True:
            window.hide()
            return True



from threading import Thread

# Remote imports
from protonvpn_cli.utils import get_config_value #noqa

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import GObject as gobject

# Local imports
from ..gui_logger import gui_logger
from ..constants import (
    TRAY_CFG_DICT, 
    TEMPLATE,
    PATH_AUTOCONNECT_SERVICE,
    SERVICE_NAME
)
from ..constants import UI_SETTINGS
from ..utils import get_gui_config, tab_style_manager

class SettingsView: 
    def __init__(self, interface, Gtk, settings_presenter, queue): 
        interface.add_from_file(UI_SETTINGS)
        self.set_objects(interface, Gtk, settings_presenter, queue)

        interface.connect_signals({
            "settings_notebook_page_changed": self.settings_notebook_page_changed,
            "update_tier_combobox_changed": self.update_tier_combobox_changed,
            "update_user_pass_button_clicked": self.update_user_pass_button_clicked,
            "tray_data_tx_combobox_changed": self.tray_data_tx_combobox_changed,
            "tray_servername_combobox_changed": self.tray_servername_combobox_changed,
            "tray_time_connected_combobox_changed": self.tray_time_connected_combobox_changed,
            "tray_serverload_combobox_changed": self.tray_serverload_combobox_changed,
            "update_autoconnect_combobox_changed": self.update_autoconnect_combobox_changed,
            "update_quick_connect_combobox_changed": self.update_quick_connect_combobox_changed,
            "custom_dns_switch_changed": self.custom_dns_switch_changed,
            "update_custom_dns_button_clicked": self.update_custom_dns_button_clicked,
            "update_protocol_combobox_changed": self.update_protocol_combobox_changed,
            "update_killswitch_switch_changed": self.update_killswitch_switch_changed,
            "update_dns_leak_switch_changed": self.update_dns_leak_switch_changed,
            "split_tunneling_switch_changed": self.split_tunneling_switch_changed,
            "update_split_tunneling_button_clicked": self.update_split_tunneling_button_clicked,
            "SettingsWindow_delete_event": self.SettingsWindow_delete_event,
            "purge_configurations_button_clicked": self.purge_configurations_button_clicked,
            "update_username_input_key_release": self.update_username_input_key_release,
            "update_password_input_key_release": self.update_password_input_key_release,
            "polkit_support_switch_changed": self.polkit_support_switch_changed,
        })

    def display_window(self):
        object_dict = {
            "general":{
                "pvpn_plan_combobox": self.pvpn_plan_combobox,
                "username": self.username_field,
                "polkit_support_switch": self.polkit_support_switch,
                "sudo_info_tooltip": self.sudo_info_tooltip
            },
            "tray_comboboxes": self.tray_dict,
            "connection":{
                "autoconnect_liststore": self.autoconnect_liststore,
                "update_autoconnect_combobox": self.update_autoconnect_combobox,
                "update_quick_connect_combobox": self.update_quick_connect_combobox,
                "update_protocol_combobox": self.update_protocol_combobox,
            },
            "advanced": {
                "dns_leak_switch":self.dns_leak_switch,
                "killswitch_switch":self.killswitch_switch,
                "split_tunneling_switch":self.split_tunneling_switch,
                "split_tunneling_list": self.split_tunneling_textview,
                "update_split_tunneling_button":self.update_split_tunneling_button,
            }
        }
        self.settings_presenter.load_configurations(object_dict)
        # self.settings_window.show()
        gobject.idle_add(self.settings_window.show)

    def set_objects(self, interface, Gtk, settings_presenter, queue):
        self.interface = interface
        self.queue = queue
        self.settings_presenter = settings_presenter
        self.settings_window = self.interface.get_object("SettingsWindow")

        # General Tab
        self.pvpn_plan_combobox = self.interface.get_object("update_tier_combobox")
        self.username_field = self.interface.get_object("update_username_input")
        self.password_field = self.interface.get_object("update_password_input")
        self.update_user_pass_button = self.interface.get_object("update_user_pass_button")
        self.polkit_support_switch = self.interface.get_object("polkit_support_switch")
        self.sudo_info_tooltip = self.interface.get_object("sudo_info_tooltip")

        # Tray tab
        self.tray_dict = {k:self.interface.get_object(k) for k,v in TRAY_CFG_DICT.items()}
        
        # Connection Tab
        self.autoconnect_liststore = interface.get_object("AutoconnectListStore")
        self.update_autoconnect_combobox = self.interface.get_object("update_autoconnect_combobox")
        self.update_quick_connect_combobox = self.interface.get_object("update_quick_connect_combobox")
        self.update_protocol_combobox = self.interface.get_object("update_protocol_combobox")

        # Advanced Tab
        self.dns_leak_switch = self.interface.get_object("update_dns_leak_switch")
        self.killswitch_switch = self.interface.get_object("update_killswitch_switch")
        self.split_tunneling_switch = self.interface.get_object("split_tunneling_switch")
        self.split_tunneling_textview = self.interface.get_object("split_tunneling_textview")
        self.update_split_tunneling_button = self.interface.get_object("update_split_tunneling_button")
        self.split_tunnel_grid = self.interface.get_object("split_tunneling_grid") 

        # DASHBOARD - Server List
        self.tree_object = self.interface.get_object("ServerTreeStore")

        self.settings_tab_dict = {
            "general_tab_style": self.interface.get_object("general_tab_label").get_style_context(), 
            "sys_tray_tab_style": self.interface.get_object("sys_tray_tab_label").get_style_context(),
            "connection_tab_style": self.interface.get_object("connection_tab_label").get_style_context(),
            "advanced_tab_style": self.interface.get_object("advanced_tab_label").get_style_context()
        }

    def settings_notebook_page_changed(self, notebook, selected_tab, actual_tab_index):
        """Updates Settings Window tab style
        """
        if actual_tab_index == 0:
            tab_style_manager("general_tab_style", self.settings_tab_dict)
        elif actual_tab_index == 1:
            tab_style_manager("sys_tray_tab_style", self.settings_tab_dict)
        elif actual_tab_index == 2:
            tab_style_manager("connection_tab_style", self.settings_tab_dict)
        elif actual_tab_index == 3:
            tab_style_manager("advanced_tab_style", self.settings_tab_dict)

    # General tab
    def update_tier_combobox_changed(self, combobox):
        tier = int(get_config_value("USER", "tier"))
        tree_iter = combobox.get_active_iter()
        if tree_iter is not None:
            model = combobox.get_model()
            selected_tier, tier_display = model[tree_iter][:2]
            if selected_tier != tier:
                self.queue.put(dict(action="display_dialog", label="Updating ProtoVPN plan...", spinner=True, hide_close_button=True))
                gui_logger.debug(">>> Starting \"update_tier_combobox_changed\" thread.")
                thread = Thread(target=self.settings_presenter.update_pvpn_plan, kwargs=dict(
                                                                tree_object=self.tree_object, 
                                                                tier=int(selected_tier+1),
                                                                tier_display=tier_display))
                thread.daemon = True
                thread.start()

    def update_username_input_key_release(self, entry, event):
        self.update_user_pass_button.set_property("sensitive", False)
        if len(entry.get_text().strip()) > 0:
            if len(self.password_field.get_text().strip()) > 0:
                self.update_user_pass_button.set_property("sensitive", True)
            else:
                self.update_user_pass_button.set_property("sensitive", False)
        
    def update_password_input_key_release(self, entry, event):
        self.update_user_pass_button.set_property("sensitive", False)
        if len(entry.get_text().strip()) > 0:
            if len(self.username_field.get_text().strip()) > 0:
                self.update_user_pass_button.set_property("sensitive", True)
            else:
                self.update_user_pass_button.set_property("sensitive", False)

    def update_user_pass_button_clicked(self, button):
        """Button/Event handler to update Username & Password
        """
        self.queue.put(dict(action="display_dialog", label="Updating username and password...", spinner=True, hide_close_button=True))
        gui_logger.debug(">>> Starting \"update_user_pass\" thread.")

        thread = Thread(target=self.settings_presenter.update_user_pass, kwargs=dict(
                                                    username=self.username_field.get_text().strip(),
                                                    password=self.password_field.get_text().strip()))
        thread.daemon = True
        thread.start()

    def polkit_support_switch_changed(self, switch, state):
        polkit_enabled = int(get_gui_config("general_tab", "polkit_enabled"))
        
        update_to = 0
        if polkit_enabled == 0:
            update_to = 1

        if (state and polkit_enabled == 0) or (not state and polkit_enabled != 0):
            self.queue.put(dict(action="display_dialog", label="Applying changes...", spinner=True, hide_close_button=True))
            
            thread = Thread(target=self.settings_presenter.on_polkit_change, args=[update_to])
            thread.daemon = True
            thread.start()

    # System tray tab
    def tray_data_tx_combobox_changed(self, combobox):
        display_data_tx = get_gui_config("tray_tab", "display_data_tx")
        tree_iter = combobox.get_active_iter()
        if tree_iter is not None:
            model = combobox.get_model()
            option, display = model[tree_iter][:2]
            if option != int(display_data_tx):
                gui_logger.debug(">>> Starting \"tray_data_tx_combobox_changed\" thread.")
                thread = Thread(target=self.settings_presenter.update_tray_display, kwargs=dict(setting_value=option, setting_display="tray_data_tx_combobox"))
                thread.daemon = True
                thread.start()

    def tray_servername_combobox_changed(self, combobox):
        display_data_tx = get_gui_config("tray_tab", "display_server")
        tree_iter = combobox.get_active_iter()
        if tree_iter is not None:
            model = combobox.get_model()
            option, display = model[tree_iter][:2]
            if option != int(display_data_tx):
                gui_logger.debug(">>> Starting \"tray_servername_combobox_changed\" thread.")
                thread = Thread(target=self.settings_presenter.update_tray_display, kwargs=dict(setting_value=option, setting_display="tray_servername_combobox"))
                thread.daemon = True
                thread.start()

    def tray_time_connected_combobox_changed(self, combobox):
        display_data_tx = get_gui_config("tray_tab", "display_time_conn")
        tree_iter = combobox.get_active_iter()
        if tree_iter is not None:
            model = combobox.get_model()
            option, display = model[tree_iter][:2]
            if option != int(display_data_tx):
                gui_logger.debug(">>> Starting \"tray_servername_combobox_changed\" thread.")
                thread = Thread(target=self.settings_presenter.update_tray_display, kwargs=dict(setting_value=option, setting_display="tray_time_connected_combobox"))
                thread.daemon = True
                thread.start()

    def tray_serverload_combobox_changed(self, combobox):
        display_data_tx = get_gui_config("tray_tab", "display_serverload")
        tree_iter = combobox.get_active_iter()
        if tree_iter is not None:
            model = combobox.get_model()
            option, display = model[tree_iter][:2]
            if option != int(display_data_tx):
                gui_logger.debug(">>> Starting \"tray_servername_combobox_changed\" thread.")
                thread = Thread(target=self.settings_presenter.update_tray_display, kwargs=dict(setting_value=option, setting_display="tray_serverload_combobox"))
                thread.daemon = True
                thread.start()

    # def tray_run_commands_combobox_changed(self, combobox):
    #     run_commands_as = int(get_gui_config("tray_tab", "run_commands_as"))
    #     tree_iter = combobox.get_active_iter()
    #     if tree_iter is not None:
    #         model = combobox.get_model()
    #         user_choice, sudo_type = model[tree_iter][:2]
    #         if user_choice != run_commands_as:
    #             self.queue.put(dict(action="display_dialog", label="Updating sudo type...", spinner=True, hide_close_button=True))
    #             gui_logger.debug(">>> Starting \"on_sudo_type\" thread.")
    #             thread = Thread(target=self.settings_presenter.on_sudo_type, kwargs=dict(
    #                                                             user_choice=user_choice,
    #                                                             sudo_type="tray_run_commands_combobox",
    #                                                             ))
    #             thread.daemon = True
    #             thread.start()
    # polkit_support_switch_changed

    # Connection tab
    def update_autoconnect_combobox_changed(self, combobox):
        autoconnect_setting = get_gui_config("conn_tab", "autoconnect")
        tree_iter = combobox.get_active_iter()
        if tree_iter is not None:
            model = combobox.get_model()
            user_choice, country_display = model[tree_iter][:2]
            if user_choice != autoconnect_setting:
                self.queue.put(dict(action="display_dialog", label="Updating autoconnect settings...", spinner=True, hide_close_button=True))
                gui_logger.debug(">>> Starting \"update_autoconnect_combobox_changed\" thread.")
                thread = Thread(target=self.settings_presenter.update_connect_preference, kwargs=dict(
                                                                user_choice=user_choice,
                                                                country_display=country_display))
                thread.daemon = True
                thread.start()

    def update_quick_connect_combobox_changed(self, combobox):
        autoconnect_setting = get_gui_config("conn_tab", "quick_connect")
        tree_iter = combobox.get_active_iter()
        if tree_iter is not None:
            model = combobox.get_model()
            user_choice, country_display = model[tree_iter][:2]

            if user_choice != autoconnect_setting:
                self.queue.put(dict(action="display_dialog", label="Updating quick connect settings...", spinner=True, hide_close_button=True))
                gui_logger.debug(">>> Starting \"update_quick_connect_combobox_changed\" thread.")

                thread = Thread(target=self.settings_presenter.update_connect_preference, kwargs=dict(
                                                                user_choice=user_choice,
                                                                country_display=country_display,
                                                                quick_connect=True))
                thread.daemon = True
                thread.start()
                
    def custom_dns_switch_changed(self, switch, state):
        pass

    def update_custom_dns_button_clicked(self, button):
        pass

    def update_protocol_combobox_changed(self, combobox):
        """Button/Event handler to update OpenVP Protocol  
        """
        autoconnect_setting = get_config_value("USER", "default_protocol")
        tree_iter = combobox.get_active_iter()
        if tree_iter is not None:
            model = combobox.get_model()
            index, protocol = model[tree_iter][:2]
            protocol = protocol.lower()
            if protocol.lower() != autoconnect_setting.lower():
                gui_logger.debug(">>> Starting \"update_protocol_combobox_changed\" thread.")
                thread = Thread(target=self.settings_presenter.update_def_protocol, args=[protocol])
                thread.daemon = True
                thread.start()   

    # Advanced tab
    def update_killswitch_switch_changed(self, switch, state):
        killswitch_protection = int(get_config_value("USER", "killswitch"))

        try:
            split_tunnel = int(get_config_value("USER", "split_tunnel"))
        except KeyError:
            gui_logger.debug("[!] Split tunneling has not been configured.")
            split_tunnel = 0

        if killswitch_protection == 0:
            update_to = 1
        else:
            update_to = 0
        
        if (state and killswitch_protection == 0) or (not state and killswitch_protection != 0):
            if update_to == 1 and split_tunnel >= 0:
                self.split_tunneling_switch.set_property('sensitive', False)
            else:
                self.split_tunneling_switch.set_property('sensitive', True)
                
            thread = Thread(target=self.settings_presenter.update_killswitch, args=[update_to])
            thread.daemon = True
            thread.start()

    def update_dns_leak_switch_changed(self, switch, state):
        dns_protection = get_config_value("USER", "dns_leak_protection")
        if dns_protection == "0":
            update_to = "1"
        elif dns_protection != "0":
            update_to = "0"

        if (state and dns_protection == "0") or (not state and dns_protection != "0"):
            thread = Thread(target=self.settings_presenter.update_dns, args=[update_to])
            thread.daemon = True
            thread.start()

    def split_tunneling_switch_changed(self, switch, state):
        killswitch_protection = int(get_config_value("USER", "killswitch"))

        try:
            split_tunnel = int(get_config_value("USER", "split_tunnel"))
        except KeyError:
            gui_logger.debug("[!] Split tunneling has not been configured.")
            split_tunnel = 0
        
        if split_tunnel == 0:
            update_to = 1
        else:
            update_to = 0

        if state:
            self.split_tunnel_grid.show()
        else:
            self.split_tunnel_grid.hide()

        if (state and split_tunnel == 0) or (not state and split_tunnel != 0):
            if update_to == 1 and killswitch_protection >= 0:
                self.killswitch_switch.set_property('sensitive', False)
            else:
                self.killswitch_switch.set_property('sensitive', True)

            thread = Thread(target=self.settings_presenter.update_split_tunneling_status, args=[update_to])
            thread.daemon = True
            thread.start()

    def update_split_tunneling_button_clicked(self, button):
        """Button/Event handler to update Split Tunneling IP list.
        """
        gui_logger.debug(">>> Starting \"update_split_tunneling\" thread.")
        
        self.queue.put(dict(action="display_dialog", label="Updating split tunneling configurations...", spinner=True, hide_close_button=True))

        buffer = self.split_tunneling_textview.get_buffer()
        split_tunneling_content = buffer.get_text(buffer.get_start_iter(), buffer.get_end_iter(), buffer)
        thread = Thread(target=self.settings_presenter.update_split_tunneling, kwargs=dict(
                                                        split_tunneling_content=split_tunneling_content))
        thread.daemon = True
        thread.start()

    # To avoid getting the ConfigurationsWindow destroyed and not being re-rendered again
    def SettingsWindow_delete_event(self, window, event):
        """On Delete handler is used to hide the window so it renders next time the dialog is called
        
        -Returns:Boolean
        - It needs to return True, otherwise the content will not re-render after closing the dialog
        """
        if window.get_property("visible") is True:
            window.hide()
            return True
            
    # To-do
    def purge_configurations_button_clicked(self, button):
        """Button/Event handler to purge configurations
        """
        gui_logger.debug(">>> Starting \"purge_configurations\" thread.")

        self.queue.put(dict(action="display_dialog", label="Purging configurations configurations...", spinner=True, hide_close_button=True))


        thread = Thread(target=self.settings_presenter.purge_configurations)
        thread.daemon = True
        thread.start()



import os
import sys
from queue import Queue

# Remote imports
from protonvpn_cli.constants import CONFIG_FILE, CONFIG_DIR #noqa
from protonvpn_cli.utils import check_root #noqa

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import  Gtk, Gdk

# Local imports
from .views.login_view import LoginView
from .views.dashboard_view import DashboardView
from .views.settings_view import SettingsView
from .views.dialog_view import DialogView

from .presenters.login_presenter import LoginPresenter
from .presenters.dashboard_presenter import DashboardPresenter
from .presenters.settings_presenter import SettingsPresenter

from .services.login_service import LoginService 
from .services.dashboard_service import DashboardService
from .services.settings_service import SettingsService

from .gui_logger import gui_logger
from .constants import (
    VERSION, 
    HELP_TEXT, 
    GUI_CONFIG_DIR, 
    GUI_CONFIG_FILE, 
    CURRDIR, 
    UI_LOGIN, 
    UI_DASHBOARD, 
    UI_SETTINGS, 
    UI_DIALOG, 
    UI_STYLES,
    CLI_ABSENCE_INFO
)

from .utils import (
    get_gui_processes,
    find_cli,
    kill_duplicate_gui_process,
    initialize_gui_config
)

def init():
    """Initializes the GUI
    """
    # if len(get_gui_processes()) > 1:
    #     gui_logger.debug("[!] Two running processes were found!")

    #     response = kill_duplicate_gui_process()

    #     if not response['success']:
    #         gui_logger.debug("[!] Unable to end previous process: {}.".format(response['message']))
    #         sys.exit(1)

    Gtk.init()
    queue = Queue()
    interface = Gtk.Builder()

    style_provider = Gtk.CssProvider()
    style_provider.load_from_path(UI_STYLES)

    Gtk.StyleContext.add_provider_for_screen(
        Gdk.Screen.get_default(),
        style_provider,
        Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION
    )

    dialog_view = DialogView(interface, Gtk, queue)
    
    if not find_cli():
        dialog_view.display_dialog(label=CLI_ABSENCE_INFO, spinner=False, hide_close_button=True)
    else:
        gui_logger.debug("\n______________________________________\n\n\tINITIALIZING NEW GUI WINDOW\n______________________________________\n")

        if not os.path.isfile(GUI_CONFIG_FILE):
            initialize_gui_config()

        if not os.path.isfile(CONFIG_FILE): 
            gui_logger.debug(">>> Loading LoginWindow")
            
            settings_service = SettingsService()
            settings_presenter = SettingsPresenter(settings_service, queue)
            settings_view = SettingsView(interface, Gtk, settings_presenter, queue)

            dashboard_service = DashboardService()
            dashboard_presenter = DashboardPresenter(dashboard_service, queue)
            dashboard_view = DashboardView(interface, Gtk, dashboard_presenter, settings_view, queue)

            login_service = LoginService()
            login_presenter = LoginPresenter(login_service, queue)
            login_view = LoginView(interface, Gtk, login_presenter, dashboard_view, queue)

            login_view.display_window()
        else:
            gui_logger.debug(">>> Loading DashboardWindow")
            settings_service = SettingsService()
            settings_presenter = SettingsPresenter(settings_service, queue)
            settings_view = SettingsView(interface, Gtk, settings_presenter, queue)

            dashboard_service = DashboardService()
            dashboard_presenter = DashboardPresenter(dashboard_service, queue)
            dashboard_view = DashboardView(interface, Gtk, dashboard_presenter, settings_view, queue)
            
            dashboard_view.display_window()
    Gtk.main()




import time
from threading import Thread
from concurrent import futures

from ..constants import UI_LOGIN, VERSION
from ..gui_logger import gui_logger

class LoginView:
    def __init__(self, interface, Gtk, login_presenter, dashboard_view, queue):
        interface.add_from_file(UI_LOGIN)
        self.set_objects(interface, Gtk, login_presenter, dashboard_view, queue)
        
        interface.connect_signals({
            "login_username_entry_key_release": self.login_username_entry_key_release,
            "login_password_entry_key_release": self.login_password_entry_key_release,
            "need_help_link_activate": self.need_help_link_activate,
            "on_login_button_clicked": self.login_button_clicked,            
        })

    def display_window(self):
        self.login_view.show()

    def set_objects(self, interface, Gtk, login_presenter, dashboard_view, queue):
        self.interface = interface
        self.login_presenter = login_presenter
        self.queue = queue
        self.dashboard_view = dashboard_view
        self.login_view = self.interface.get_object("LoginWindow")

        self.login_username_label = self.interface.get_object("login_username_label")
        self.login_password_label = self.interface.get_object("login_password_label")

        self.username_field = self.interface.get_object('username_field')
        self.password_field = self.interface.get_object('password_field')

        self.member_free_radio =  self.interface.get_object('member_free')
        self.member_basic_radio =  self.interface.get_object('member_basic')
        self.member_plus_radio =  self.interface.get_object('member_plus')
        self.member_visionary_radio =  self.interface.get_object('member_visionary')

        self.login_button =  self.interface.get_object('login_button')

        self.popover = self.interface.get_object("login_window_popover")

        self.version_label = interface.get_object("login_window_version_label")
        self.version_label.set_markup("v.{}".format(VERSION))

    def login_username_entry_key_release(self, entry, event):
        self.login_username_label.set_markup("")

        self.login_button.set_property("sensitive", False)
        if len(entry.get_text().strip()) > 0:
            # self.login_username_label.show()
            self.login_username_label.set_markup("ProtonVPN (OpenVPN/IKEv2) Username")

            if len(self.password_field.get_text().strip()) > 0:
                self.login_button.set_property("sensitive", True)
            else:
                self.login_button.set_property("sensitive", False)
        
    def login_password_entry_key_release(self, entry, event):
        self.login_password_label.set_markup("")
        
        self.login_button.set_property("sensitive", False)
        if len(entry.get_text().strip()) > 0:
            # self.login_password_label.show()
            self.login_password_label.set_markup("ProtonVPN (OpenVPN/IKEv2) Password")

            if len(self.username_field.get_text().strip()) > 0:
                self.login_button.set_property("sensitive", True)
            else:
                self.login_button.set_property("sensitive", False)

    def need_help_link_activate(self, label, link):
        self.popover.show()

    def login_button_clicked(self, button):
        """Button/Event handler to intialize user account. Calls populate_server_list(server_tree_store) to populate server list.
        """     
        # Queue has to be used
        self.queue.put(dict(action="display_dialog", label="Intializing profile...", spinner=True, hide_close_button=True))
        
        with futures.ThreadPoolExecutor(max_workers=1) as executor:
            var_dict = dict(
                        username_field=self.username_field.get_text().strip(), 
                        password_field=self.password_field.get_text().strip(),
                        member_free_radio=self.member_free_radio.get_active(),
                        member_basic_radio=self.member_basic_radio.get_active(),
                        member_plus_radio=self.member_plus_radio.get_active(),
                        member_visionary_radio=self.member_visionary_radio .get_active()
            )
            future = executor.submit(self.login_presenter.on_login, **var_dict)
            return_value = future.result()
            if return_value:
                self.login_view.hide()
                self.dashboard_view.display_window()



import re
import os
import time
import shutil
import subprocess
import collections

# Remote imports
from protonvpn_cli.constants import CONFIG_DIR, PASSFILE, SPLIT_TUNNEL_FILE, USER #noqa
from protonvpn_cli.utils import get_config_value, is_valid_ip, set_config_value #noqa
from protonvpn_cli.connection import disconnect as pvpn_disconnect
from protonvpn_cli.country_codes import country_codes

# Local imports
from ..gui_logger import gui_logger
from ..constants import (
    TRAY_CFG_DICT, 
    TEMPLATE,
    PATH_AUTOCONNECT_SERVICE,
    SERVICE_NAME,
    GUI_CONFIG_DIR,
    TRAY_SUDO_TYPES
)
from ..utils import (
    set_gui_config,
    get_gui_config,
    find_cli,
    is_polkit_installed
)

class SettingsPresenter:
    def __init__(self,  settings_service, queue):
        self.settings_service = settings_service
        self.queue = queue

    def update_user_pass(self, **kwargs):
        """Function that updates username and password.
        """
        username = kwargs.get("username")
        password = kwargs.get("password")
        
        result_bool, display_message = self.settings_service.set_user_pass(username, password)
        
        self.queue.put(dict(action="update_dialog", label=display_message))
       
        return result_bool

    def update_dns(self, dns_value):
        """Function that updates DNS settings. It either enables or disables.
        """
        return_val = False
        display_message = "Could not update DNS."

        if self.settings_service.set_dns(dns_value):
            display_message = "DNS Management updated."
            return_val = True

        gui_logger.debug(">>> Ended tasks in \"dns_leak_switch_clicked\" thread. Result {}".format(display_message))

        return return_val

    def update_pvpn_plan(self, **kwargs):
        """Function that updates ProtonVPN plan.
        """
        gui_logger.debug(">>> Running \"set_protonvpn_tier\".")

        protonvpn_plan = kwargs.get("tier")
        return_val = False

        display_message = "Unable to update ProtonVPN Plan!"
        if self.settings_service.set_pvpn_tier(protonvpn_plan):
            display_message = "ProtonVPN Plan has been updated to <b>{}</b>!\nPlease refresh servers!".format(kwargs.get("tier_display"))
            return_val = True
        
        self.queue.put(dict(action="update_dialog", label=display_message))

        gui_logger.debug(">>> Ended tasks in \"set_protonvpn_tier\" thread. Result:{}".format("ProtonVPN Plan has been updated!"))   

        return return_val

    def update_def_protocol(self, openvpn_protocol):
        """Function that updates default protocol.
        """
        gui_logger.debug(">>> Running \"set_default_protocol\".")

        return_val = True
        if not self.settings_service.set_default_protocol(openvpn_protocol):
            return_val = False
            gui_logger.debug(">>> Could not update default protocol.")   

        gui_logger.debug(">>> Ended tasks in \"set_default_protocol\" thread.")   

        return return_val

    def update_connect_preference(self, **kwargs):
        """Function that updates autoconnect. 
        """
        active_choice = kwargs.get("user_choice")
        return_val = False
        response = ''

        gui_logger.debug(">>> Running \"update_connect_preference\".")

        display_message = ""
        if not "quick_connect" in kwargs:
            response_bool, display_message = self.settings_service.set_autoconnect(active_choice)
        else:
            response_bool = self.settings_service.set_quickconnect(active_choice)

        
        if response_bool:
            if display_message:
                display_message = display_message+"\n"

            display_message = "{}{} setting updated to <b>{}</b>!".format(display_message, "Autoconnect" if not "quick_connect" in kwargs else "Quick connect", kwargs.get("country_display"))
            return_val = True

        gui_logger.debug(">>> Result: {} <-> {}".format(response_bool, display_message))

        self.queue.put(dict(action="update_dialog", label=display_message))

        gui_logger.debug(">>> Ended tasks in \"update_autoconnect\" thread.") 

        return return_val

    def update_killswitch(self, update_to):
        """Function that updates killswitch configurations. 
        """
        display_message = "Unable to update killswitch configuration!"
        return_val = False

        if self.settings_service.set_killswitch(update_to):
            display_message = ">>> Kill Switch configuration updated to {}".format("enabled" if update_to == "1" else "disabled")
            return_val = True

        gui_logger.debug(">>> Ended tasks in \"update_killswitch_switch_changed\" thread. Result: \"{0}\"".format(display_message)) 

        return return_val  

    def update_split_tunneling_status(self, update_to):
        """Updates Split tunneling status. Should be invoked by the Split Tunneling switch.
        """
        return_val = False

        if update_to == "1":
            result = "Split tunneling has been <b>enabled</b>!\n"
        else:
            result = "Split tunneling has been <b>disabled</b>!\n"

        if int(get_config_value("USER", "killswitch")):
            result = result + "Split Tunneling <b>can't</b> be used with Kill Switch, Kill Switch has been <b>disabled</b>!\n\n"
            time.sleep(1)

        display_message = "Unable to update split tunneling status!"
        if not self.settings_service.set_split_tunneling(update_to):
            result = display_message
            return_val = True

        gui_logger.debug(">>> Ended tasks in \"set_split_tunnel\" thread. Result: \"{0}\"".format(result)) 

        return return_val

    def update_split_tunneling(self, **kwargs):
        """Function that updates split tunneling configurations.
        """
        gui_logger.debug(">>> Running \"set_split_tunnel\".")
        
        return_val = False
        split_tunneling_content = kwargs.get("split_tunneling_content")
        result = "Split tunneling configurations <b>updated</b>!\n"
        disabled_ks = False
        invalid_ip = False

        ip_list = self.settings_service.reformat_ip_list(split_tunneling_content)
        
        valid_ips = self.settings_service.check_valid_ips(ip_list)
        
        if not type(valid_ips) == bool and len(valid_ips) > 1 and not valid_ips[0]:
            result = "<b>{0}</b> is not valid!\nNone of the IP's were added, please try again with a different IP.".format(valid_ips[1])
            gui_logger.debug("[!] Invalid IP \"{0}\".".format(valid_ips[1]))
            invalid_ip = True

        if not invalid_ip:
            if len(ip_list) == 0:
                result = "unable to disable Split Tunneling !\n\n"
                if self.settings_service.set_split_tunneling(0):
                    result = "Split tunneling <b>disabled</b>!\n\n"
                    return_val = True
            else:
                result = result + "The following servers were added:\n\n{}".format([ip for ip in ip_list])
                return_val = True
                
                if int(get_config_value("USER", "killswitch")):
                    result = "Split Tunneling <b>can't</b> be used with Kill Switch.\nKill Switch could not be disabled, cancelling update!"
                    if self.settings_service.set_killswitch(0):
                        result = result + "Split Tunneling <b>can't</b> be used with Kill Switch.\nKill Switch has been <b>disabled</b>!\n\n"
                        disabled_ks = True
        
                if not disabled_ks and not self.settings_service.set_split_tunneling_ips(ip_list):
                    result = "Unable to add IPs to Split Tunneling file!"
                    return_val = False
                
        self.queue.put(dict(action="update_dialog", label=result))

        gui_logger.debug(">>> Ended tasks in \"set_split_tunnel\" thread. Result: \"{0}\"".format(result))   
        
        return return_val

    def update_tray_display(self, **kwargs):
        """Function to update what the tray should display.
        """    
        gui_logger.debug(">>> Running \"tray_configurations\".")
        
        return_val = False
        setting_value = kwargs.get("setting_value")
        setting_display = kwargs.get("setting_display")
        display_message = ''

        if "serverload" in setting_display:
            display_message = "server load"
        elif "server" in setting_display:
            display_message = "server name"
        elif "data" in setting_display:
            display_message = "data transmission"
        elif "time" in setting_display:
            display_message = "time connected"

        result = "Unable to update {} to {}!".format(display_message, "displayed" if setting_value == 1 else "hidden")
        if self.settings_service.set_tray_display_setting(setting_display, setting_value):
            result = "Tray {0} is <b>{1}</b>!".format(display_message, "displayed" if setting_value == 1 else "hidden")
            return_val = True

        gui_logger.debug(">>> Ended tasks in \"tray_configurations\" thread. Result: \"{0}\"".format(result))   
        
        return return_val

    def on_polkit_change(self, update_to):
        result_bool, display_message = self.settings_service.update_polkit(update_to)

        self.queue.put(dict(action="update_dialog", label=display_message))

    def purge_configurations(self):
        """Function to purge all current configurations.
        """
        # To-do: Confirm prior to allowing user to do this
        gui_logger.debug(">>> Running \"set_split_tunnel\".")

        config_removed = False
        gui_removed = False
        
        pvpn_disconnect(passed=True)

        if os.path.isdir(CONFIG_DIR):
            shutil.rmtree(CONFIG_DIR)
            gui_logger.debug(">>> Result: \"{0}\"".format("Configurations purged."))
            config_removed = True

        if os.path.isdir(GUI_CONFIG_DIR):
            shutil.rmtree(GUI_CONFIG_DIR)
            gui_logger.debug(">>> Result: \"{0}\"".format("GUI configurations purged."))
            gui_removed = True

        self.queue.put(dict(action="update_dialog", label="Configurations purged!"))

        gui_logger.debug(">>> Ended tasks in \"set_split_tunnel\" thread.")   

        return config_removed == gui_removed


    def load_configurations(self, object_dict):
        """Function that loads user configurations before showing the configurations window.
        """
        self.load_general_settings(object_dict["general"])
        self.load_tray_settings(object_dict["tray_comboboxes"])
        self.load_connection_settings(object_dict["connection"])
        self.load_advanced_settings(object_dict["advanced"])

    def load_general_settings(self, general_settings_dict):
        polkit_support_switch = general_settings_dict["polkit_support_switch"]
        sudo_info_tooltip = general_settings_dict["sudo_info_tooltip"]
        setter = 0

        tooltip_msg = "Could not find PolKit installed on your system. For more information, please visit: \nhttps://github.com/ProtonVPN/linux-gui"

        username = get_config_value("USER", "username")
        tier = int(get_config_value("USER", "tier"))

        # Populate username
        general_settings_dict["username"].set_text(username)   
        # Set tier
        general_settings_dict["pvpn_plan_combobox"].set_active(tier)

        polkit_support_switch.set_property('sensitive', False)

        if is_polkit_installed():
            if self.settings_service.polkit == 1:
                polkit_support_switch.set_state(True)
            
            polkit_support_switch.set_property('sensitive', True)
            use_cases = "\n-Update username and password (root protected file)\n-Enable/disable autoconnect (create/remove .service file)\n-Connect/disconnect to/from VPN (run CLI commands)"
            tooltip_msg = "Displays a window to enter sudo password, which is needed for the following cases:{}\n\nIt is recommended to enabled this if you don't want to use the GUI via the terminal.".format(use_cases)
        
        sudo_info_tooltip.set_tooltip_text(tooltip_msg)

    def load_tray_settings(self, display_dict):
        # Load tray configurations
        for k,v in TRAY_CFG_DICT.items(): 
            setter = 0
            try: 
                setter = int(get_gui_config("tray_tab", v))
            except KeyError:
                gui_logger.debug("[!] Unable to find {} key.".format(v))
                set_gui_config("tray_tab", v, 0)

            combobox = display_dict[k]
            combobox.set_active(setter)

    def load_connection_settings(self, object_dict):
        # Set Autoconnect on boot combobox 
        autoconnect_liststore =  object_dict["autoconnect_liststore"]
        update_autoconnect_combobox = object_dict["update_autoconnect_combobox"]
        update_quick_connect_combobox = object_dict["update_quick_connect_combobox"]
        update_protocol_combobox = object_dict["update_protocol_combobox"]

        server_list = self.populate_autoconnect_list(autoconnect_liststore, return_list=True)

        #Get values
        try:
            autoconnect_setting = get_gui_config("conn_tab", "autoconnect")
        except (KeyError, IndexError):
            autoconnect_setting = 0
        try:
            quick_connect_setting = get_gui_config("conn_tab", "quick_connect")
        except (KeyError, IndexError):
            quick_connect = 0 
        default_protocol = get_config_value("USER", "default_protocol")

        # Get indexes
        try:
            autoconnect_index = list(server_list.keys()).index(autoconnect_setting)
        except ValueError:
            autoconnect_index = 0

        try:
            quick_connect_index = list(server_list.keys()).index(quick_connect_setting)
        except ValueError:
            quick_connect_index = 0

        # Set values
        update_autoconnect_combobox.set_active(autoconnect_index)
        update_quick_connect_combobox.set_active(quick_connect_index)

        if default_protocol == "tcp":
            update_protocol_combobox.set_active(0)
        else:
            update_protocol_combobox.set_active(1)

    def load_advanced_settings(self, object_dict):
        # User values
        dns_leak_protection = get_config_value("USER", "dns_leak_protection")
        custom_dns = get_config_value("USER", "custom_dns")
        killswitch = get_config_value("USER", "killswitch")

        try:
            split_tunnel = get_config_value("USER", "split_tunnel")
        except (KeyError, IndexError):
            split_tunnel = '0'

        # Object
        dns_leak_switch = object_dict["dns_leak_switch"]
        killswitch_switch = object_dict["killswitch_switch"]
        split_tunneling_switch = object_dict["split_tunneling_switch"]
        split_tunneling_list = object_dict["split_tunneling_list"]

        # Set DNS Protection
        if dns_leak_protection == '1':
        # if dns_leak_protection == '1' or (dns_leak_protection != '1' and custom_dns.lower() != "none"):
            dns_leak_switch.set_state(True)
        else:
            dns_leak_switch.set_state(False)

        # Populate Split Tunelling
        # Check if killswtich is != 0, if it is then disable split tunneling Function
        if killswitch != '0':
            killswitch_switch.set_state(True)
            split_tunneling_switch.set_property('sensitive', False)
        else:
            killswitch_switch.set_state(False)

        if split_tunnel != '0':
            split_tunneling_switch.set_state(True)
            killswitch_switch.set_property('sensitive', False)
            if killswitch != '0':
                split_tunneling_list.set_property('sensitive', False)
                object_dict["update_split_tunneling_button"].set_property('sensitive', False)
                
            split_tunneling_buffer = split_tunneling_list.get_buffer()
            content = ""
            try:
                with open(SPLIT_TUNNEL_FILE) as f:
                    lines = f.readlines()

                    for line in lines:
                        content = content + line

                    split_tunneling_buffer.set_text(content)
            except FileNotFoundError:
                split_tunneling_buffer.set_text(content)
        else:
            split_tunneling_switch.set_state(False)  

    def populate_autoconnect_list(self, autoconnect_liststore, return_list=False):
        """Function that populates autoconnect dropdown list.
        """
        autoconnect_alternatives = self.settings_service.generate_autoconnect_list()
        other_choice_dict = {
            "dis": "Disabled",
            "fast": "Fastest",
            "rand": "Random", 
            "p2p": "Peer2Peer", 
            "sc": "Secure Core (Plus/Visionary)",
            "tor": "Tor (Plus/Visionary)"
        }
        return_values = collections.OrderedDict()

        for alt in autoconnect_alternatives:
            if alt in other_choice_dict:
                # if return_list:
                return_values[alt] = other_choice_dict[alt]
                # else:
                autoconnect_liststore.append([alt, other_choice_dict[alt], alt])
            else:
                for k,v in country_codes.items():
                    if alt.lower() == v.lower():
                        # if return_list:
                        return_values[k] = v
                        # else:
                        autoconnect_liststore.append([k, v, k])
        
        if return_list:
            return return_values



import os
import getpass
USER = getpass.getuser()

VERSION = "2.1.1"
GITHUB_URL_RELEASE = "https://github.com/ProtonVPN/linux-gui/releases/latest"

# GUI configurations
GUI_CONFIG_DIR = os.path.join(os.path.expanduser("~{0}".format(USER)), ".pvpn-gui")
GUI_CONFIG_FILE = os.path.join(GUI_CONFIG_DIR, "pvpn-gui.cfg")

CURRDIR = os.path.dirname(os.path.abspath(__file__))

UI_LOGIN = os.path.join(CURRDIR, "resources/ui/login_window.glade")
UI_DASHBOARD = os.path.join(CURRDIR, "resources/ui/dashboard_window.glade")
UI_SETTINGS = os.path.join(CURRDIR, "resources/ui/settings_window.glade")
UI_DIALOG = os.path.join(CURRDIR, "resources/ui/message_dialog.glade")
UI_STYLES = os.path.join(CURRDIR, "resources/styles/main.css")

LARGE_FLAGS_BASE_PATH = os.path.join(CURRDIR, "resources/img/flags/large/")
SMALL_FLAGS_BASE_PATH = os.path.join(CURRDIR, "resources/img/flags/small/")
FEATURES_BASE_PATH = os.path.join(CURRDIR, "resources/img/utils/")

# Tray configuration naming
TRAY_CFG_SERVERLOAD = "display_serverload"
TRAY_CFG_SERVENAME = "display_server"
TRAY_CFG_DATA_TX = "display_data_tx"
TRAY_CFG_TIME_CONN = "display_time_conn"
TRAY_CFG_DICT = {
    "tray_data_tx_combobox": TRAY_CFG_DATA_TX,
    "tray_servername_combobox": TRAY_CFG_SERVENAME,
    "tray_time_connected_combobox": TRAY_CFG_TIME_CONN,
    "tray_serverload_combobox": TRAY_CFG_SERVERLOAD
}
TRAY_CFG_SUDO = "run_commands_as"
TRAY_SUDO_TYPES = {
    "tray_run_commands_combobox": TRAY_CFG_SUDO,
}

SERVICE_NAME = "custompvpn-autoconnect" 
PATH_AUTOCONNECT_SERVICE = "/etc/systemd/system/{}.service".format(SERVICE_NAME)
TEMPLATE ="""
[Unit]
Description=Custom ProtonVPN-CLI auto-connect
Wants=network-online.target

[Service]
Type=forking
ExecStart=PATH
ExecStop=STOP
Environment=PVPN_WAIT=300
Environment=PVPN_DEBUG=1
Environment=SUDO_USER=user

[Install]
WantedBy=multi-user.target
"""
CLI_ABSENCE_INFO = """
<b>Could not find linux-cli installed on your system!</b>\t
Original linux-cli is needed for the GUI to work.

<b>Install via pip:</b>
sudo pip3 install protonvpn-cli

<b>Install via Github:</b>
git clone https://github.com/protonvpn/linux-cli
cd protonvpn-cli-ng
sudo python3 setup.py install
"""
HELP_TEXT = """
<b>To-do</b>
"""

